import os

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.preprocessing_layer import (
    PreprocessingLayer,
)
from keras_hub.src.utils.preset_utils import ASSET_DIR
from keras_hub.src.utils.preset_utils import TOKENIZER_CONFIG_FILE
from keras_hub.src.utils.preset_utils import builtin_presets
from keras_hub.src.utils.preset_utils import find_subclass
from keras_hub.src.utils.preset_utils import get_file
from keras_hub.src.utils.preset_utils import get_preset_loader
from keras_hub.src.utils.preset_utils import get_preset_saver
from keras_hub.src.utils.python_utils import classproperty
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export(
    [
        "keras_hub.models.Tokenizer",
        "keras_hub.tokenizers.Tokenizer",
    ]
)
class Tokenizer(PreprocessingLayer):
    """A base class for tokenizer layers.

    Tokenizers in the KerasHub library should all subclass this layer.
    The class provides two core methods `tokenize()` and `detokenize()` for
    going from plain text to sequences and back. A tokenizer is a subclass of
    `keras.layers.Layer` and can be combined into a `keras.Model`.

    Subclassers should always implement the `tokenize()` method, which will also
    be the default when calling the layer directly on inputs.

    Subclassers can optionally implement the `detokenize()` method if the
    tokenization is reversible. Otherwise, this can be skipped.

    Subclassers should implement `get_vocabulary()`, `vocabulary_size()`,
    `token_to_id()` and `id_to_token()` if applicable. For some simple
    "vocab free" tokenizers, such as a whitespace splitter show below, these
    methods do not apply and can be skipped.

    Example:

    ```python
    class WhitespaceSplitterTokenizer(keras_hub.tokenizers.Tokenizer):
        def tokenize(self, inputs):
            return tf.strings.split(inputs)

        def detokenize(self, inputs):
            return tf.strings.reduce_join(inputs, separator=" ", axis=-1)

    tokenizer = WhitespaceSplitterTokenizer()

    # Tokenize some inputs.
    tokenizer.tokenize("This is a test")

    # Shorthard for `tokenize()`.
    tokenizer("This is a test")

    # Detokenize some outputs.
    tokenizer.detokenize(["This", "is", "a", "test"])
    ```
    """

    backbone_cls = None

    def __init__(self, *args, **kwargs):
        self.config_file = kwargs.pop("config_file", TOKENIZER_CONFIG_FILE)
        super().__init__(*args, **kwargs)
        self.file_assets = None

    def tokenize(self, inputs, *args, **kwargs):
        """Transform input tensors of strings into output tokens.

        Args:
            inputs: Input tensor, or dict/list/tuple of input tensors.
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.
        """
        raise NotImplementedError(
            "No implementation of `tokenize()` was found for "
            f"{self.__class__.__name__}. All tokenizers should implement "
            "`tokenize()`."
        )

    def detokenize(self, inputs, *args, **kwargs):
        """Transform tokens back into strings.

        Args:
            inputs: Input tensor, or dict/list/tuple of input tensors.
            *args: Additional positional arguments.
            **kwargs: Additional keyword arguments.
        """
        raise NotImplementedError(
            "No implementation of `detokenize()` was found for "
            f"{self.__class__.__name__}."
        )

    def get_vocabulary(self):
        """Get the tokenizer vocabulary as a list of strings terms."""
        raise NotImplementedError(
            "No implementation of `get_vocabulary()` was found for "
            f"{self.__class__.__name__}."
        )

    def vocabulary_size(self):
        """Returns the total size of the token id space."""
        raise NotImplementedError(
            "No implementation of `vocabulary_size()` was found for "
            f"{self.__class__.__name__}."
        )

    def id_to_token(self, id):
        """Convert an integer id to a string token."""
        raise NotImplementedError(
            "No implementation of `id_to_token()` was found for "
            f"{self.__class__.__name__}."
        )

    def token_to_id(self, token):
        """Convert a string token to an integer id."""
        raise NotImplementedError(
            "No implementation of `token_to_id()` was found for "
            f"{self.__class__.__name__}."
        )

    @property
    def special_tokens(self):
        """List all built-in special tokens for the tokenizer."""
        if not hasattr(self, "_special_token_attrs"):
            return []
        tokens = set(getattr(self, a) for a in self._special_token_attrs)
        return list(tokens)

    @property
    def special_token_ids(self):
        """List all built-in special token ids for the tokenizer."""
        if not hasattr(self, "_special_token_attrs"):
            return []
        ids = set(getattr(self, f"{a}_id") for a in self._special_token_attrs)
        if None in ids:
            raise ValueError(
                "Cannot access `special_token_ids` before a vocabulary has "
                "been set on the tokenizer."
            )
        return list(ids)

    def _add_special_token(self, token, name):
        if not hasattr(self, "_special_token_attrs"):
            self._special_token_attrs = []
        self._special_token_attrs.append(name)
        setattr(self, name, token)
        try:
            id = self.token_to_id(token)
        except (ValueError, AttributeError):
            id = None
        setattr(self, f"{name}_id", id)

    def _update_special_token_ids(self):
        if not hasattr(self, "_special_token_attrs"):
            return
        vocabulary = self.get_vocabulary()
        for attr in set(self._special_token_attrs):
            token = getattr(self, attr)
            if token not in vocabulary:
                classname = self.__class__.__name__
                raise ValueError(
                    f"Cannot find special token `'{token}'` in the provided "
                    f"vocabulary for `{classname}`. Please ensure `'{token}'` "
                    "is in the provided vocabulary when creating the Tokenizer."
                )
        for attr in self._special_token_attrs:
            token = getattr(self, attr)
            setattr(self, f"{attr}_id", self.token_to_id(token))

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "config_file": self.config_file,
            }
        )
        return config

    def save_to_preset(self, preset_dir):
        """Save tokenizer to a preset directory.

        Args:
            preset_dir: The path to the local model preset directory.
        """
        saver = get_preset_saver(preset_dir)
        saver.save_tokenizer(self)

    @preprocessing_function
    def call(self, inputs, *args, training=None, **kwargs):
        return self.tokenize(inputs, *args, **kwargs)

    def load_preset_assets(self, preset):
        asset_path = None
        for asset in self.file_assets:
            subdir = self.config_file.split(".")[0]
            preset_path = os.path.join(ASSET_DIR, subdir, asset)
            asset_path = get_file(preset, preset_path)
        tokenizer_config_file = os.path.dirname(asset_path)
        self.load_assets(tokenizer_config_file)

    @classproperty
    def presets(cls):
        """List built-in presets for a `Tokenizer` subclass."""
        return builtin_presets(cls)

    @classmethod
    def from_preset(
        cls,
        preset,
        config_file=TOKENIZER_CONFIG_FILE,
        **kwargs,
    ):
        """Instantiate a `keras_hub.models.Tokenizer` from a model preset.

        A preset is a directory of configs, weights and other file assets used
        to save and load a pre-trained model. The `preset` can be passed as
        one of:

        1. a built-in preset identifier like `'bert_base_en'`
        2. a Kaggle Models handle like `'kaggle://user/bert/keras/bert_base_en'`
        3. a Hugging Face handle like `'hf://user/bert_base_en'`
        4. a path to a local preset directory like `'./bert_base_en'`

        For any `Tokenizer` subclass, you can run `cls.presets.keys()` to list
        all built-in presets available on the class.

        This constructor can be called in one of two ways. Either from the base
        class like `keras_hub.models.Tokenizer.from_preset()`, or from
        a model class like `keras_hub.models.GemmaTokenizer.from_preset()`.
        If calling from the base class, the subclass of the returning object
        will be inferred from the config in the preset directory.

        Args:
            preset: string. A built-in preset identifier, a Kaggle Models
                handle, a Hugging Face handle, or a path to a local directory.
            load_weights: bool. If `True`, the weights will be loaded into the
                model architecture. If `False`, the weights will be randomly
                initialized.

        Examples:
        ```python
        # Load a preset tokenizer.
        tokenizer = keras_hub.tokenizer.Tokenizer.from_preset("bert_base_en")

        # Tokenize some input.
        tokenizer("The quick brown fox tripped.")

        # Detokenize some input.
        tokenizer.detokenize([5, 6, 7, 8, 9])
        ```
        """
        loader = get_preset_loader(preset)
        backbone_cls = loader.check_backbone_class()
        if cls.backbone_cls != backbone_cls:
            cls = find_subclass(preset, cls, backbone_cls)
        return loader.load_tokenizer(cls, config_file, **kwargs)

    def export_to_transformers(self, path):
        """Export the tokenizer to HuggingFace Transformers format.

        This saves tokenizer assets in a format compatible with HuggingFace
        Transformers.

        Args:
            path: str. Path to save the exported tokenizer.
        """
        from keras_hub.src.utils.transformers.export.hf_exporter import (
            export_tokenizer,
        )

        export_tokenizer(self, path)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.qwen_moe.qwen_moe_backbone import QwenMoeBackbone
from keras_hub.src.tokenizers.byte_pair_tokenizer import BytePairTokenizer


@keras_hub_export(
    "keras_hub.tokenizers.QwenMoeTokenizer",
)
class QwenMoeTokenizer(BytePairTokenizer):
    """Tokenizer for Qwen Moe model.

    This tokenizer implements byte-pair encoding (BPE) for Qwen models,
    handling special tokens like BOS (beginning of sequence) and EOS (end of
    sequence).

    Args:
        vocabulary: Dictionary mapping tokens to token IDs, or path to
            vocabulary file.
        merges: List of BPE merges, or path to merges file.
        bos_token: Beginning of sequence token. Defaults to None.
        eos_token: End of sequence token. Defaults to "<|endoftext|>".
        misc_special_tokens: Set of additional special tokens. Defaults to
            empty set.
    """

    backbone_cls = QwenMoeBackbone

    def __init__(
        self,
        vocabulary=None,
        merges=None,
        **kwargs,
    ):
        # Add EOS token
        eos_token = "<|endoftext|>"
        self._add_special_token(eos_token, "end_token")

        self.start_token_id = None
        self.start_token = None
        self.pad_token_id = 0

        super().__init__(
            vocabulary=vocabulary,
            merges=merges,
            **kwargs,
        )



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.seq_2_seq_lm import Seq2SeqLM
from keras_hub.src.models.t5gemma.t5gemma_backbone import T5GemmaBackbone
from keras_hub.src.models.t5gemma.t5gemma_seq_2_seq_lm_preprocessor import (
    T5GemmaSeq2SeqLMPreprocessor,
)
from keras_hub.src.utils.tensor_utils import any_equal


@keras_hub_export("keras_hub.models.T5GemmaSeq2SeqLM")
class T5GemmaSeq2SeqLM(Seq2SeqLM):
    """An end-to-end T5Gemma model for seq2seq language modeling.

    A seq2seq language model (LM) is an encoder-decoder model which is used for
    conditional text generation. The encoder is given a "context" text (fed to
    the encoder), and the decoder predicts the next token based on both the
    encoder inputs and the previous tokens. You can finetune `T5GemmaSeq2SeqLM`
    to generate text for any seq2seq task (e.g., translation or summarization).

    This model has a `generate()` method, which generates text based on a
    prompt. The generation strategy used is controlled by an additional
    `sampler` argument on `compile()`. You can recompile the model with
    different `keras_hub.samplers` objects to control the generation. By
    default, `"greedy"` sampling will be used.

    This model can optionally be configured with a `preprocessor` layer, in
    which case it will automatically apply preprocessing to string inputs during
    `fit()`, `predict()`, `evaluate()` and `generate()`. This is done by default
    when creating the model with `from_preset()`.

    Args:
        backbone: A `keras_hub.models.T5GemmaBackbone` instance.
        preprocessor: A `keras_hub.models.T5GemmaSeq2SeqLMPreprocessor` or
            `None`. If `None`, this model will not apply preprocessing, and
            inputs should be preprocessed before calling the model. Defaults
            to `None`.

    Examples:

    Use `generate()` to do text generation.
    ```python
    import numpy as np
    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM.from_preset(
        "t5gemma_b_b_prefixlm_it"
    )
    # Generate with encoder-only input.
    t5gemma_lm.generate("The quick brown fox jumped.", max_length=30)

    # Generate with batched encoder-only inputs.
    t5gemma_lm.generate(
        ["The quick brown fox jumped.", "The whale."],
        max_length=30
    )
    # Generate with encoder and decoder inputs.
    t5gemma_lm.generate(
        {
            "encoder_text": "The quick brown fox jumped.",
            "decoder_text": "A fast fox"
        },
        max_length=30
    )
    ```

    Compile the `generate()` function with a custom sampler.
    ```python
    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM.from_preset(
        "t5gemma_b_b_prefixlm_it"
    )
    t5gemma_lm.compile(sampler="top_k")
    t5gemma_lm.generate("I want to say", max_length=30)

    t5gemma_lm.compile(sampler=keras_hub.samplers.BeamSampler(num_beams=2))
    t5gemma_lm.generate("I want to say", max_length=30)
    ```

    Use `generate()` without preprocessing.
    ```python
    # Preprocessed inputs, with encoder inputs corresponding to
    # "The quick brown fox", and the decoder inputs to "A fast fox".
    # Use `"padding_mask"` to indicate values that should not be overridden.
    prompt = {
        "encoder_token_ids": np.array([[2, 10, 133, 2119, 6219, 23602, 1, 0]]),
        "encoder_padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 0]]),
        "decoder_token_ids": np.array([[2, 133, 1769, 1, 0, 0, 0]]),
        "decoder_padding_mask": np.array([[1, 1, 1, 1, 0, 0, 0]])
    }

    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM.from_preset(
        "t5gemma_b_b_prefixlm_it",
        preprocessor=None,
    )
    t5gemma_lm.generate(prompt)
    ```

    Call `fit()` on a single batch.
    ```python
    features = {
        "encoder_text": ["The quick fox jumped.", "I forgot my homework."],
        "decoder_text": ["The fast hazel fox leapt.", "I forgot my assignment."]
    }
    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM.from_preset(
        "t5gemma_b_b_prefixlm_it"
    )
    t5gemma_lm.fit(x=features, batch_size=2)
    ```

    Call `fit()` without preprocessing.
    ```python
    x = {
        "encoder_token_ids": np.array([[2, 133, 2119, 1, 0]] * 2),
        "encoder_padding_mask": np.array([[1, 1, 1, 1, 0]] * 2),
        "decoder_token_ids": np.array([[2, 133, 1769, 1, 0]] * 2),
        "decoder_padding_mask": np.array([[1, 1, 1, 1, 1]] * 2),
    }
    y = np.array([[133, 1769, 1, 0, 0]] * 2)
    sw = np.array([[1, 1, 1, 0, 0]] * 2)

    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM.from_preset(
        "t5gemma_b_b_prefixlm_it",
        preprocessor=None,
    )
    t5gemma_lm.fit(x=x, y=y, sample_weight=sw, batch_size=2)
    ```

    Custom backbone and vocabulary.
    ```python
    features = {
        "encoder_text": ["The quick fox jumped.", "I forgot my homework."],
        "decoder_text": ["The fast hazel fox leapt.", "I forgot my assignment."]
    }
    tokenizer = keras_hub.models.T5GemmaTokenizer(
        proto="proto.spm",
    )
    preprocessor = keras_hub.models.T5GemmaSeq2SeqLMPreprocessor(
        tokenizer=tokenizer,
        encoder_sequence_length=128,
        decoder_sequence_length=128,
    )
    backbone = keras_hub.models.T5GemmaBackbone(
        vocabulary_size=32000,
        # Encoder parameters.
        encoder_hidden_dim=256,
        encoder_intermediate_dim=512,
        encoder_num_layers=4,
        encoder_num_attention_heads=4,
        encoder_num_key_value_heads=2,
        encoder_head_dim=64,
        encoder_layer_types=["full_attention"] * 4,
        # Decoder parameters.
        decoder_hidden_dim=256,
        decoder_intermediate_dim=512,
        decoder_num_layers=4,
        decoder_num_attention_heads=4,
        decoder_num_key_value_heads=2,
        decoder_head_dim=64,
        decoder_layer_types=["full_attention"] * 4,
        # Common parameters.
        dropout_rate=0.1,
        rms_norm_eps=1e-6,
        query_pre_attn_scalar=1.0,
        attention_bias=False,
        hidden_activation="gelu_approximate",
    )
    t5gemma_lm = keras_hub.models.T5GemmaSeq2SeqLM(
        backbone=backbone,
        preprocessor=preprocessor,
    )
    t5gemma_lm.fit(x=features, batch_size=2)
    ```
    """

    backbone_cls = T5GemmaBackbone
    preprocessor_cls = T5GemmaSeq2SeqLMPreprocessor

    def __init__(self, backbone, preprocessor=None, **kwargs):
        # === Layers ===
        self.backbone = backbone
        self.preprocessor = preprocessor

        # === Functional Model ===
        # This must be "backbone.input" i.e. the full input structure,
        # rather than "backbone.inputs" which is the flattened list of inputs.
        inputs = backbone.input
        sequence_output = backbone(inputs)["decoder_sequence_output"]
        logits = backbone.decoder_token_embedding(sequence_output, reverse=True)
        if self.backbone.final_logit_softcapping is not None:
            logits = logits / self.backbone.final_logit_softcapping
            logits = keras.ops.tanh(logits)
            logits = logits * self.backbone.final_logit_softcapping
        super().__init__(
            inputs=inputs,
            outputs=logits,
            **kwargs,
        )

    def call_encoder(self, token_ids, padding_mask):
        """Process inputs through the encoder stack."""
        encoder_embeddings = self.backbone.token_embedding(token_ids)
        encoder_embeddings *= keras.ops.cast(
            keras.ops.sqrt(self.backbone.encoder_hidden_dim),
            encoder_embeddings.dtype,
        )
        encoder_hidden_states = self.backbone.encoder_dropout(
            encoder_embeddings, training=False
        )
        for layer in self.backbone.encoder_layers:
            encoder_hidden_states = layer(
                encoder_hidden_states, padding_mask=padding_mask, training=False
            )
        encoder_output = self.backbone.encoder_norm(encoder_hidden_states)
        encoder_output = self.backbone.encoder_dropout(
            encoder_output, training=False
        )
        return encoder_output, padding_mask

    def call_decoder_with_cache(
        self,
        decoder_token_ids,
        decoder_padding_mask,
        cache,
        cache_update_index,
        encoder_output,
        encoder_padding_mask,
    ):
        """Forward pass of `T5GemmaSeq2SeqLM`'s decoder with cache.

        `call_decoder_with_cache` adds an additional forward pass for the model
        for autoregressive inference. Unlike calling the model directly, this
        method allows caching previous key/value Tensors in the attention
        layers, and avoids recomputing the outputs of seen tokens.

        Args:
            decoder_token_ids: A dense int Tensor with shape
                `(batch_size, max_length)`. The token ids for the decoder.
            decoder_padding_mask: A dense int Tensor with shape `(batch_size,
                max_length)`. The padding mask for the decoder.
            cache: A dense float Tensor, the cache of key and value states.
            cache_update_index: int, or int Tensor. The index of the current
                token being processed in the whole sequence.
            encoder_output: A dense float Tensor. The output of the encoder.
            encoder_padding_mask: A dense int Tensor. The padding mask for
                the encoder output.

        Returns:
            A `(logits, hidden_states, cache)` tuple. Where `logits` is the
            language model logits for the input token_ids, `hidden_states` is
            the final hidden representation of the input tokens, and `cache` is
            the updated decoding cache.
        """
        self_attention_cache, cross_attention_cache = cache
        hidden_states = self.backbone.decoder_token_embedding(decoder_token_ids)
        hidden_states *= keras.ops.cast(
            keras.ops.sqrt(self.backbone.decoder_hidden_dim),
            hidden_states.dtype,
        )
        hidden_states = self.backbone.decoder_dropout(
            hidden_states, training=False
        )
        # Every decoder layer has a separate cache for the self-attention layer
        # and the cross-attention layer. We update all of them separately.
        updated_self_attention_caches = []
        updated_cross_attention_caches = []
        for i, layer in enumerate(self.backbone.decoder_layers):
            layer_self_cache = (
                self_attention_cache[:, i, ...]
                if self_attention_cache is not None
                else None
            )
            layer_cross_cache = (
                cross_attention_cache[:, i, ...]
                if cross_attention_cache is not None
                else None
            )
            layer_cache = (layer_self_cache, layer_cross_cache)
            hidden_states, updated_layer_cache = layer(
                (hidden_states, encoder_output),
                self_attention_padding_mask=decoder_padding_mask,
                cross_attention_padding_mask=encoder_padding_mask,
                cache=layer_cache,
                cache_update_index=cache_update_index,
                training=False,
            )
            new_self_cache, new_cross_cache = updated_layer_cache
            updated_self_attention_caches.append(new_self_cache)
            updated_cross_attention_caches.append(new_cross_cache)
        self_attention_cache = keras.ops.stack(
            updated_self_attention_caches, axis=1
        )
        cross_attention_cache = keras.ops.stack(
            updated_cross_attention_caches, axis=1
        )
        hidden_states = self.backbone.decoder_norm(hidden_states)
        logits = self.backbone.decoder_token_embedding(
            hidden_states, reverse=True
        )
        if self.backbone.final_logit_softcapping is not None:
            logits = logits / self.backbone.final_logit_softcapping
            logits = keras.ops.tanh(logits)
            logits = logits * self.backbone.final_logit_softcapping
        return (
            logits,
            hidden_states,
            (self_attention_cache, cross_attention_cache),
        )

    def _build_cache(
        self,
        encoder_token_ids,
        encoder_padding_mask,
        decoder_token_ids,
        decoder_padding_mask,
    ):
        """Build an empty cache for use with `call_with_cache()`."""
        encoder_output, encoder_padding_mask = self.call_encoder(
            encoder_token_ids, encoder_padding_mask
        )
        batch_size = keras.ops.shape(decoder_token_ids)[0]
        num_layers = self.backbone.decoder_num_layers
        num_kv_heads = self.backbone.decoder_num_key_value_heads
        head_dim = self.backbone.decoder_head_dim
        self_cache_shape = (
            batch_size,
            num_layers,
            2,
            keras.ops.shape(decoder_token_ids)[1],
            num_kv_heads,
            head_dim,
        )
        self_attention_cache = keras.ops.zeros(
            self_cache_shape, dtype=self.compute_dtype
        )
        cross_attention_cache = None
        _, hidden_states, cache = self.call_decoder_with_cache(
            decoder_token_ids=decoder_token_ids,
            decoder_padding_mask=decoder_padding_mask,
            cache=(self_attention_cache, cross_attention_cache),
            cache_update_index=0,
            encoder_output=encoder_output,
            encoder_padding_mask=encoder_padding_mask,
        )
        extra_cache_info = (encoder_output, encoder_padding_mask)
        return hidden_states, cache, extra_cache_info

    def generate_step(self, inputs, stop_token_ids=None):
        """A compilable generation function for a single batch of inputs.

        This function represents the inner, XLA-compilable, generation function
        for a single batch of inputs. Inputs should have the same structure as
        model inputs, a dictionary with keys `"token_ids"` and `"padding_mask"`.
        `"encoder_token_ids"`, `"encoder_padding_mask"`, `"decoder_token_ids"`
        and `"decoder_padding_mask"`.

        Args:
            inputs: A dictionary with four keys - `"encoder_token_ids"`,
                `"encoder_padding_mask"`, `"decoder_token_ids"` and
                `"decoder_padding_mask"`, with batched tensor values.
            stop_token_ids: Tuple of id's of end token's to stop on. If all
                sequences have produced a new stop token, generation
                will stop.
        """
        encoder_token_ids = inputs["encoder_token_ids"]
        encoder_padding_mask = inputs["encoder_padding_mask"]
        decoder_token_ids = inputs["decoder_token_ids"]
        decoder_padding_mask = inputs["decoder_padding_mask"]
        # Create and seed cache with a single forward pass.
        hidden_states, cache, extra_cache_info = self._build_cache(
            encoder_token_ids=encoder_token_ids,
            encoder_padding_mask=encoder_padding_mask,
            decoder_token_ids=decoder_token_ids,
            decoder_padding_mask=decoder_padding_mask,
        )
        encoder_output, encoder_padding_mask = extra_cache_info
        # Compute the lengths of all user inputted tokens ids.
        row_lengths = keras.ops.sum(
            keras.ops.cast(decoder_padding_mask, "int32"), axis=-1
        )
        # Start at the first index that has no user inputted id.
        index = keras.ops.min(row_lengths)

        def next(prompt, cache, index):
            # The cache index is the index of our previous token.
            cache_update_index = index - 1
            batch_size = keras.ops.shape(prompt)[0]
            prompt = keras.ops.slice(
                prompt, [0, cache_update_index], [batch_size, 1]
            )
            (
                logits,
                _,
                updated_cache,
            ) = self.call_decoder_with_cache(
                decoder_token_ids=prompt,
                decoder_padding_mask=None,
                cache_update_index=cache_update_index,
                cache=cache,
                encoder_output=encoder_output,
                encoder_padding_mask=encoder_padding_mask,
            )
            return keras.ops.squeeze(logits, axis=1), None, updated_cache

        decoder_token_ids = self.sampler(
            next=next,
            prompt=decoder_token_ids,
            cache=cache,
            index=index,
            mask=decoder_padding_mask,
            stop_token_ids=stop_token_ids,
            hidden_states=hidden_states,
            model=self,
        )

        # Compute an output padding mask with the token ids we updated.
        if stop_token_ids is not None:
            # Build a mask of `stop_token_ids` locations not in the original
            # prompt (not in locations where `decoder_padding_mask` is True).
            end_locations = any_equal(
                decoder_token_ids,
                stop_token_ids,
                keras.ops.logical_not(decoder_padding_mask),
            )
            # Use cumsum to get ones in all locations after end_locations.
            end_locations = keras.ops.cast(end_locations, "int32")
            cumsum = keras.ops.cast(
                keras.ops.cumsum(end_locations, axis=-1), "int32"
            )
            overflow = cumsum - end_locations
            # Our padding mask is the inverse of these overflow locations.
            decoder_padding_mask = keras.ops.logical_not(
                keras.ops.cast(overflow, "bool")
            )
        else:
            # Without early stopping, all locations will have been updated.
            decoder_padding_mask = keras.ops.ones_like(
                decoder_token_ids, dtype="bool"
            )

        return {
            "decoder_token_ids": decoder_token_ids,
            "decoder_padding_mask": decoder_padding_mask,
        }



from keras_hub.src.models.qwen_moe.qwen_moe_backbone import QwenMoeBackbone
from keras_hub.src.models.qwen_moe.qwen_moe_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, QwenMoeBackbone)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.causal_lm_preprocessor import CausalLMPreprocessor
from keras_hub.src.models.qwen3_moe.qwen3_moe_backbone import Qwen3MoeBackbone
from keras_hub.src.models.qwen3_moe.qwen3_moe_tokenizer import Qwen3MoeTokenizer


@keras_hub_export(
    "keras_hub.models.Qwen3MoeCausalLMPreprocessor",
)
class Qwen3MoeCausalLMPreprocessor(CausalLMPreprocessor):
    backbone_cls = Qwen3MoeBackbone
    tokenizer_cls = Qwen3MoeTokenizer



import keras
from keras import initializers
from keras import ops


class RoformerNorm(keras.layers.Layer):
    """A normalization layer for Roformer that implements RMS normalization."""

    def __init__(self, epsilon=1e-6, **kwargs):
        super().__init__(**kwargs)
        self.epsilon = epsilon

    def build(self, input_shape):
        dim = input_shape[-1]
        self.scale = self.add_weight(
            name="scale",
            trainable=True,
            shape=(dim,),
            initializer="ones",
            dtype=self.variable_dtype,
        )
        self.built = True

    def call(self, x):
        x = ops.cast(x, "float32")
        var = ops.mean(ops.power(x, 2), axis=-1, keepdims=True)
        x = x * ops.rsqrt(var + self.epsilon)
        return ops.cast(x * self.scale, self.compute_dtype)

    def get_config(self):
        config = super().get_config()
        config.update({"epsilon": self.epsilon})
        return config


class RoformrPositionalEmbedding(keras.layers.Layer):
    """Native rotary implement by jianlin su
    from native implement
    https://github.com/bojone/bert4keras

    """

    def __init__(self, output_dim, max_wavelength=10000, **kwargs):
        super().__init__(**kwargs)
        self.max_wavelength = max_wavelength
        self.output_dim = output_dim

    def call(self, tensors):
        input_shape = ops.shape(tensors[0])
        seq_len = input_shape[1]
        position_ids = ops.arange(0, seq_len, dtype=tensors[0].dtype)[None]
        embeddings = self.sinusoidal_embeddings(
            position_ids, self.output_dim, self.max_wavelength
        )
        embeddings = ops.cast(embeddings, self.compute_dtype)

        ndim = ops.ndim(tensors[0])
        sinusoidal = self.align(embeddings, [0, 1, -1], ndim)
        cos_pos = ops.repeat(sinusoidal[..., 1::2], 2, -1)
        sin_pos = ops.repeat(sinusoidal[..., ::2], 2, -1)
        outputs = []
        for tensor in tensors:
            tensor2 = ops.stack([-tensor[..., 1::2], tensor[..., ::2]], ndim)
            tensor2 = ops.reshape(tensor2, ops.shape(tensor))
            outputs.append(tensor * cos_pos + tensor2 * sin_pos)
        return outputs[0] if len(outputs) == 1 else outputs

    def align(self, tensor, axes, ndim=None):
        ndim = ndim or max(axes) + 1
        indices = [None] * ndim
        for i in axes:
            indices[i] = slice(None)
        if keras.config.backend() == "jax":
            return tensor[tuple(indices)]
        return tensor[indices]

    def sinusoidal_embeddings(self, pos, dim, base=10000):
        if dim % 2 != 0:
            raise ("Dimension must be even")

        indices = ops.arange(0, dim // 2, dtype="float32")
        indices = ops.power(ops.cast(base, dtype="float32"), -2 * indices / dim)
        embeddings = ops.einsum("...,d->...d", pos, indices)
        embeddings = ops.stack(
            [ops.sin(embeddings), ops.cos(embeddings)], axis=-1
        )
        shape = list(ops.shape(embeddings))
        embeddings = ops.reshape(embeddings, shape[:-2] + [-1])
        return embeddings

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "out_dim": self.out_dim,
                "max_wavelength": self.max_wavelength,
            }
        )
        return config


@keras.saving.register_keras_serializable(package="keras_hub")
class RoformerAttention(keras.layers.Layer):
    """MultiHeadAttention by roformerV2

    modifity from native implement
    https://github.com/bojone/bert4keras
    """

    def __init__(
        self,
        heads,
        head_size,
        out_dim=None,
        use_bias=False,
        max_wavelength=10000,
        kernel_initializer="glorot_uniform",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.heads = heads
        self.head_size = head_size
        self.out_dim = out_dim or heads * head_size
        self.use_bias = use_bias
        self.kernel_initializer = initializers.get(kernel_initializer)
        self.max_wavelength = max_wavelength

    def build(self, input_shape):
        super().build(input_shape)
        self.q_dense = keras.layers.Dense(
            units=self.head_size * self.heads,
            use_bias=self.use_bias,
            kernel_initializer=self.kernel_initializer,
            name="q_dense_layer",
            dtype=self.dtype_policy,
        )
        self.q_dense.build(input_shape)

        self.k_dense = keras.layers.Dense(
            units=self.head_size * self.heads,
            use_bias=self.use_bias,
            kernel_initializer=self.kernel_initializer,
            name="k_dense_layer",
            dtype=self.dtype_policy,
        )
        self.k_dense.build(input_shape)

        self.v_dense = keras.layers.Dense(
            units=self.head_size * self.heads,
            use_bias=self.use_bias,
            kernel_initializer=self.kernel_initializer,
            name="v_dense_layer",
            dtype=self.dtype_policy,
        )
        self.v_dense.build(input_shape)

        self.o_dense = keras.layers.Dense(
            units=self.out_dim,
            use_bias=self.use_bias,
            kernel_initializer=self.kernel_initializer,
            name="o_dense_layer",
            dtype=self.dtype_policy,
        )
        self.o_dense.build([None, None, self.head_size * self.heads])

        self.rotary_embedding_layer = RoformrPositionalEmbedding(
            self.head_size, self.max_wavelength, dtype=self.dtype_policy
        )
        self.rotary_embedding_layer.build([])

    def call(self, x, attention_mask=None):
        qw = self.q_dense(x)
        kw = self.k_dense(x)
        vw = self.v_dense(x)

        b, s = ops.shape(qw)[:2]
        qw = ops.reshape(qw, (b, s, self.heads, self.head_size))
        kw = ops.reshape(kw, (b, s, self.heads, self.head_size))
        vw = ops.reshape(vw, (b, s, self.heads, self.head_size))

        qw, kw = self.rotary_embedding_layer([qw, kw])
        flash_attention = keras.config.is_flash_attention_enabled()
        attention_mask = ops.reshape(attention_mask, [b, 1, s, 1])
        if keras.config.backend() == "torch":
            attention_mask = ops.repeat(attention_mask, s, -1)
            attention_mask = ops.transpose(attention_mask, [0, 1, 3, 2])
        o = ops.dot_product_attention(
            qw, kw, vw, mask=attention_mask, flash_attention=flash_attention
        )

        return self.o_dense(ops.reshape(o, [b, s, -1]))

    def compute_output_shape(self, input_shape):
        return input_shape

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "heads": self.heads,
                "head_size": self.head_size,
                "out_dim": self.out_dim,
                "use_bias": self.use_bias,
                "max_wavelength": self.max_wavelength,
                "kernel_initializer": initializers.serialize(
                    self.kernel_initializer
                ),
            }
        )
        return config



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.models.mobilenetv5.mobilenetv5_backbone import (
    MobileNetV5Backbone,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_image_classifier_preprocessor import (  # noqa: E501
    MobileNetV5ImageClassifierPreprocessor,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_layers import ConvNormAct
from keras_hub.src.models.mobilenetv5.mobilenetv5_utils import (
    SelectAdaptivePool2d,
)
from keras_hub.src.models.task import Task


@keras_hub_export("keras_hub.models.MobileNetV5ImageClassifier")
class MobileNetV5ImageClassifier(ImageClassifier):
    """An end-to-end MobileNetV5 model for image classification.

    This model attaches a classification head to a `MobileNetV5Backbone`.
    The head consists of a global pooling layer, an optional convolutional
    head, a dropout layer, and a final dense classifier layer.

    This model can optionally be configured with a `preprocessor` layer, in
    which case it will automatically apply preprocessing to image inputs during
    `fit()`, `predict()`, and `evaluate()`.

    Args:
        backbone: A `keras_hub.models.MobileNetV5Backbone` instance.
        num_classes: int. The number of classes for the classification head.
        preprocessor: A `keras_hub.models.ImageClassifierPreprocessor` or
            `None`. If `None`, this model will not apply preprocessing.
        head_hidden_size: int. The number of channels in the convolutional
            head.
        global_pool: str. The type of global pooling to use.
        drop_rate: float. The dropout rate for the head.
        head_dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to
            use for the head computations and weights.

    Example:
    ```python
    import keras
    from keras_hub.models import MobileNetV5Backbone
    from keras_hub.models import MobileNetV5ImageClassifier

    # Randomly initialized task model with a custom config.
    model_args = {
        "stackwise_block_types": [["er"], ["uir", "uir"]],
        "stackwise_num_blocks": [1, 2],
        "stackwise_num_filters": [[24], [48, 48]],
        "stackwise_strides": [[2], [2, 1]],
        "stackwise_act_layers": [["relu"], ["relu", "relu"]],
        "stackwise_exp_ratios": [[4.0], [6.0, 6.0]],
        "stackwise_se_ratios": [[0.0], [0.0, 0.0]],
        "stackwise_dw_kernel_sizes": [[0], [5, 5]],
        "stackwise_dw_start_kernel_sizes": [[0], [0, 0]],
        "stackwise_dw_end_kernel_sizes": [[0], [0, 0]],
        "stackwise_exp_kernel_sizes": [[3], [0, 0]],
        "stackwise_pw_kernel_sizes": [[1], [0, 0]],
        "stackwise_num_heads": [[0], [0, 0]],
        "stackwise_key_dims": [[0], [0, 0]],
        "stackwise_value_dims": [[0], [0, 0]],
        "stackwise_kv_strides": [[0], [0, 0]],
        "stackwise_use_cpe": [[False], [False, False]],
        "use_msfa": False,
    }
    backbone = MobileNetV5Backbone(**model_args)
    model = MobileNetV5ImageClassifier(backbone, 1000)
    images = keras.ops.ones((1, 224, 224, 3))
    output = model.predict(images)
    ```
    """

    backbone_cls = MobileNetV5Backbone
    preprocessor_cls = MobileNetV5ImageClassifierPreprocessor

    def __init__(
        self,
        backbone,
        num_classes,
        preprocessor=None,
        head_hidden_size=2048,
        global_pool="avg",
        drop_rate=0.0,
        head_dtype=None,
        **kwargs,
    ):
        head_dtype = head_dtype or backbone.dtype_policy
        data_format = getattr(backbone, "data_format", "channels_last")

        # === Layers ===
        self.backbone = backbone
        self.preprocessor = preprocessor
        if backbone.use_msfa:
            self.global_pool = SelectAdaptivePool2d(
                pool_type=global_pool, data_format=data_format, flatten=True
            )
            self.conv_head = None
            self.flatten = None
        else:
            self.global_pool = SelectAdaptivePool2d(
                pool_type=global_pool, data_format=data_format, flatten=False
            )
            self.conv_head = ConvNormAct(
                out_chs=head_hidden_size,
                kernel_size=1,
                pad_type="same",
                norm_layer=backbone.norm_layer,
                act_layer=backbone.act_layer,
                bias=False,
                name="conv_head",
                dtype=head_dtype,
            )
            self.flatten = keras.layers.Flatten(dtype=head_dtype)
        self.dropout = (
            keras.layers.Dropout(drop_rate, dtype=head_dtype)
            if drop_rate > 0.0
            else None
        )
        self.classifier = (
            keras.layers.Dense(num_classes, dtype=head_dtype, name="classifier")
            if num_classes > 0
            else keras.layers.Activation("linear", name="identity_classifier")
        )

        # === Functional Model ===
        inputs = self.backbone.input
        x = self.backbone(inputs)
        x = self.global_pool(x)
        if self.conv_head is not None:
            x = self.conv_head(x)
        if self.flatten is not None:
            x = self.flatten(x)
        if self.dropout is not None:
            x = self.dropout(x)
        outputs = self.classifier(x)
        Task.__init__(self, inputs=inputs, outputs=outputs, **kwargs)

        # === Config ===
        self.num_classes = num_classes
        self.head_hidden_size = head_hidden_size
        self.global_pool_type = global_pool
        self.drop_rate = drop_rate

    def get_config(self):
        config = Task.get_config(self)
        config.update(
            {
                "num_classes": self.num_classes,
                "head_hidden_size": self.head_hidden_size,
                "global_pool": self.global_pool_type,
                "drop_rate": self.drop_rate,
            }
        )
        return config



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.distil_bert.distil_bert_backbone import (
    DistilBertBackbone,
)
from keras_hub.src.tokenizers.word_piece_tokenizer import WordPieceTokenizer


@keras_hub_export(
    [
        "keras_hub.tokenizers.DistilBertTokenizer",
        "keras_hub.models.DistilBertTokenizer",
    ]
)
class DistilBertTokenizer(WordPieceTokenizer):
    """A DistilBERT tokenizer using WordPiece subword segmentation.

    This tokenizer class will tokenize raw strings into integer sequences and
    is based on `keras_hub.tokenizers.WordPieceTokenizer`. Unlike the
    underlying tokenizer, it will check for all special tokens needed by
    DistilBERT models and provides a `from_preset()` method to automatically
    download a matching vocabulary for a DistilBERT preset.

    If input is a batch of strings (rank > 0), the layer will output a
    `tf.RaggedTensor` where the last dimension of the output is ragged.

    If input is a scalar string (rank == 0), the layer will output a dense
    `tf.Tensor` with static shape `[None]`.

    Args:
        vocabulary: A list of strings or a string filename path. If
            passing a list, each element of the list should be a single word
            piece token string. If passing a filename, the file should be a
            plain text file containing a single word piece token per line.
        lowercase: If `True`, the input text will be first lowered before
            tokenization.
        special_tokens_in_strings: bool. A bool to indicate if the tokenizer
            should expect special tokens in input strings that should be
            tokenized and mapped correctly to their ids. Defaults to False.

    Examples:

    ```python
    # Unbatched input.
    tokenizer = keras_hub.models.DistilBertTokenizer.from_preset(
        "distil_bert_base_en_uncased",
    )
    tokenizer("The quick brown fox jumped.")

    # Batched input.
    tokenizer(["The quick brown fox jumped.", "The fox slept."])

    # Detokenization.
    tokenizer.detokenize(tokenizer("The quick brown fox jumped."))

    # Custom vocabulary.
    vocab = ["[UNK]", "[CLS]", "[SEP]", "[PAD]", "[MASK]"]
    vocab += ["The", "quick", "brown", "fox", "jumped", "."]
    tokenizer = keras_hub.models.DistilBertTokenizer(vocabulary=vocab)
    tokenizer("The quick brown fox jumped.")
    ```
    """

    backbone_cls = DistilBertBackbone

    def __init__(
        self,
        vocabulary,
        lowercase=False,
        **kwargs,
    ):
        self._add_special_token("[CLS]", "cls_token")
        self._add_special_token("[SEP]", "sep_token")
        self._add_special_token("[PAD]", "pad_token")
        self._add_special_token("[MASK]", "mask_token")
        # Also add `tokenizer.start_token` and `tokenizer.end_token` for
        # compatibility with other tokenizers.
        self._add_special_token("[CLS]", "start_token")
        self._add_special_token("[SEP]", "end_token")
        super().__init__(
            vocabulary=vocabulary,
            lowercase=lowercase,
            **kwargs,
        )



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_classifier_preprocessor import (
    ImageClassifierPreprocessor,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_backbone import (
    MobileNetV5Backbone,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_image_converter import (
    MobileNetV5ImageConverter,
)


@keras_hub_export("keras_hub.models.MobileNetV5ImageClassifierPreprocessor")
class MobileNetV5ImageClassifierPreprocessor(ImageClassifierPreprocessor):
    backbone_cls = MobileNetV5Backbone
    image_converter_cls = MobileNetV5ImageConverter



from keras_hub.src.models.dinov2.dinov2_backbone import DINOV2Backbone
from keras_hub.src.models.dinov2.dinov2_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, DINOV2Backbone)



import keras
from keras import layers

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.depth_anything.depth_anything_layers import (
    DepthAnythingDepthEstimationHead,
)
from keras_hub.src.models.depth_anything.depth_anything_layers import (
    DepthAnythingNeck,
)
from keras_hub.src.models.dinov2 import DINOV2Backbone
from keras_hub.src.utils.keras_utils import standardize_data_format


@keras_hub_export("keras_hub.models.DepthAnythingBackbone")
class DepthAnythingBackbone(Backbone):
    """DepthAnything core network with hyperparameters.

    DepthAnything offers a powerful monocular depth estimation as described in
    [Depth Anything V2](https://arxiv.org/abs/2406.09414).

    The default constructor gives a fully customizable, randomly initialized
    DepthAnything model with any number of layers, heads, and embedding
    dimensions by providing the DINOV2 as the `image_encoder`. To load preset
    architectures and weights, use the `from_preset` constructor.

    Args:
        image_encoder: The DINOV2 image encoder for encoding the input images.
        reassemble_factors: List of float. The reassemble factor for each
            feature map from the image encoder. The length of the list must be
            equal to the number of feature maps from the image encoder.
        neck_hidden_dims: int. The size of the neck hidden state.
        fusion_hidden_dim: int. The size of the fusion hidden state.
        head_hidden_dim: int. The size of the neck hidden state.
        head_in_index: int. The index to select the feature from the neck
            features as the input to the head.
        feature_keys: List of string. The keys to select the feature maps from
            the image encoder. If `None`, all feature maps from the image
            encoder will be used. Defaults to `None`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done a float32 precision regardless of dtype.

    Example:
    ```python
    # Pretrained DepthAnything model.
    input_data = {
        "images": np.ones(shape=(1, 518, 518, 3), dtype="float32"),
    }
    model = keras_hub.models.DepthAnythingBackbone.from_preset(
        "depth_anything_v2_small"
    )
    model(input_data)

    # Pretrained DepthAnything model with custom image shape.
    input_data = {
        "images": np.ones(shape=(1, 224, 224, 3), dtype="float32"),
    }
    model = keras_hub.models.DepthAnythingBackbone.from_preset(
        "depth_anything_v2_small", image_shape=(224, 224, 3)
    )
    model(input_data)

    # Randomly initialized DepthAnything model with custom config.
    image_encoder = keras_hub.models.DINOV2Backbone(
        patch_size=14,
        num_layers=4,
        hidden_dim=32,
        num_heads=2,
        intermediate_dim=128,
        image_shape=(224, 224, 3),
        position_embedding_shape=(518, 518),
    )
    model = keras_hub.models.DepthAnythingBackbone(
        image_encoder=image_encoder,
        reassemble_factors=[4, 2, 1, 0.5],
        neck_hidden_dims=[16, 32, 64, 128],
        fusion_hidden_dim=128,
        head_hidden_dim=16,
        head_in_index=-1,
        feature_keys=["Stage1", "Stage2", "Stage3", "Stage4"],
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        image_encoder,
        reassemble_factors,
        neck_hidden_dims,
        fusion_hidden_dim,
        head_hidden_dim,
        head_in_index,
        feature_keys=None,
        data_format=None,
        dtype=None,
        **kwargs,
    ):
        if not isinstance(image_encoder, DINOV2Backbone):
            raise ValueError(
                "`image_encoder` must be a `DINOV2Backbone`. "
                f"Received image_encoder={image_encoder} "
                f"(of type {type(image_encoder)})."
            )
        if feature_keys is not None:
            feature_keys = [str(key) for key in feature_keys]
            for key in feature_keys:
                if key not in image_encoder.pyramid_outputs:
                    raise ValueError(
                        "All `feature_keys` must be in "
                        "`image_encoder.pyramid_outputs`. "
                        f"Received feature_keys={feature_keys}, but "
                        "`image_encoder.pyramid_outputs` contains "
                        f"{list(image_encoder.pyramid_outputs.keys())}."
                    )
        else:
            feature_keys = list(image_encoder.pyramid_outputs.keys())
        if len(reassemble_factors) != len(feature_keys):
            raise ValueError(
                "The length of `reassemble_factors` must be equal to the "
                "length of `feature_keys`. "
                f"Received len(reassemble_factors)={len(reassemble_factors)}, "
                f"len(feature_keys)={len(feature_keys)}."
            )
        data_format = standardize_data_format(data_format)
        patch_size = image_encoder.patch_size
        backbone_hidden_dim = image_encoder.hidden_dim
        image_shape = image_encoder.image_shape
        if data_format == "channels_last":
            image_size = (image_shape[0], image_shape[1])
        else:
            image_size = (image_shape[1], image_shape[2])

        # === Layers ===
        pyramid_outputs = {
            key: value
            for key, value in image_encoder.pyramid_outputs.items()
            if key in feature_keys
        }
        self.feature_extractor = keras.Model(
            inputs=image_encoder.inputs,
            outputs=pyramid_outputs,
        )
        self.feature_extractor.dtype_policy = image_encoder.dtype_policy
        self.neck = DepthAnythingNeck(
            patch_size=patch_size,
            image_size=image_size,
            backbone_hidden_dim=backbone_hidden_dim,
            neck_hidden_dims=neck_hidden_dims,
            reassemble_factors=reassemble_factors,
            fusion_hidden_dim=fusion_hidden_dim,
            num_cls_tokens=1,
            num_register_tokens=image_encoder.num_register_tokens,
            data_format=data_format,
            dtype=dtype,
            name="neck",
        )
        self.head = DepthAnythingDepthEstimationHead(
            patch_size=patch_size,
            patch_height=image_size[0] // patch_size,
            patch_width=image_size[1] // patch_size,
            fusion_hidden_dim=fusion_hidden_dim,
            head_hidden_dim=head_hidden_dim,
            head_in_index=head_in_index,
            data_format=data_format,
            dtype=dtype,
            name="head",
        )

        # === Functional Model ===
        image_input = layers.Input(shape=image_shape, name="images")
        features = self.feature_extractor(image_input)
        features = self.neck(list(features.values()))
        depth_output = self.head(features)
        super().__init__(
            inputs=image_input,
            outputs=depth_output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.image_encoder = image_encoder
        self.reassemble_factors = reassemble_factors
        self.neck_hidden_dims = neck_hidden_dims
        self.fusion_hidden_dim = fusion_hidden_dim
        self.head_hidden_dim = head_hidden_dim
        self.head_in_index = head_in_index
        self.feature_keys = feature_keys

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_encoder": layers.serialize(self.image_encoder),
                "reassemble_factors": self.reassemble_factors,
                "neck_hidden_dims": self.neck_hidden_dims,
                "fusion_hidden_dim": self.fusion_hidden_dim,
                "head_hidden_dim": self.head_hidden_dim,
                "head_in_index": self.head_in_index,
                "feature_keys": self.feature_keys,
            }
        )
        return config

    @classmethod
    def from_config(cls, config, custom_objects=None):
        config = config.copy()

        # Propagate `dtype` to `image_encoder` if needed.
        if "dtype" in config and config["dtype"] is not None:
            dtype_config = config["dtype"]
            if "dtype" not in config["image_encoder"]["config"]:
                config["image_encoder"]["config"]["dtype"] = dtype_config

        # We expect submodels to be instantiated.
        config["image_encoder"] = layers.deserialize(
            config["image_encoder"], custom_objects=custom_objects
        )
        return cls(**config)



from keras_hub.src.models.llama.llama_backbone import LlamaBackbone
from keras_hub.src.models.llama.llama_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, LlamaBackbone)



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.start_end_packer import StartEndPacker
from keras_hub.src.models.preprocessor import Preprocessor
from keras_hub.src.utils.tensor_utils import preprocessing_function
from keras_hub.src.utils.tensor_utils import strip_to_ragged


@keras_hub_export("keras_hub.models.CausalLMPreprocessor")
class CausalLMPreprocessor(Preprocessor):
    """Base class for causal language modeling preprocessing layers.

    `CausalLMPreprocessor` tasks wrap a `keras_hub.tokenizer.Tokenizer` to
    create a preprocessing layer for causal language modeling tasks. It is
    intended to be paired with a `keras.models.CausalLM` task.

    All `CausalLMPreprocessor` take inputs a single input. This can be a single
    string or a batch of strings. See examples below. These inputs
    will be tokenized and padded/truncated to a fixed sequence length.

    This layer will always output a `(x, y, sample_weight)` tuple, where `x`
    is a dictionary with the tokenized inputs, `y` contains the tokens from `x`
    offset by 1, and `sample_weight` marks where `y` contains padded
    values. The exact contents of `x` will vary depending on the model being
    used.

    a `CausalLMPreprocessor` contains two extra methods, `generate_preprocess`
    and `generate_postprocess` for use with generation. See examples below.

    All `CausalLMPreprocessor` tasks include a `from_preset()` constructor
    which can be used to load a pre-trained config and vocabularies. You can
    call the `from_preset()` constructor directly on this base class, in which
    case the correct class for you model will be automatically instantiated.

    Examples.
    ```python
    preprocessor = keras_hub.models.CausalLMPreprocessor.from_preset(
        "bert_base_en_uncased",
        sequence_length=256, # Optional.
    )

    # Tokenize, mask and pack a single sentence.
    x = "The quick brown fox jumped."
    x, y, sample_weight = preprocessor(x)

    # Tokenize and pad/truncate a batch of labeled sentences.
    x = ["The quick brown fox jumped.", "Call me Ishmael."]
    x, y, sample_weight = preprocessor(x)

    # With a `tf.data.Dataset`.
    ds = tf.data.Dataset.from_tensor_slices(x)
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)

    # Generate preprocess and postprocess.
    x = preprocessor.generate_preprocess(x)  # Tokenized numeric inputs.
    x = preprocessor.generate_postprocess(x)  # Detokenized string outputs.
    ```
    """

    def __init__(
        self,
        tokenizer,
        sequence_length=1024,
        add_start_token=True,
        add_end_token=True,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.tokenizer = tokenizer
        self.packer = None
        self.sequence_length = sequence_length
        self.add_start_token = add_start_token
        self.add_end_token = add_end_token

    def build(self, input_shape):
        # Defer packer creation to `build()` so that we can be sure tokenizer
        # assets have loaded when restoring a saved model.
        self.packer = StartEndPacker(
            start_value=self.tokenizer.start_token_id,
            end_value=self.tokenizer.end_token_id,
            pad_value=self.tokenizer.pad_token_id,
            sequence_length=self.sequence_length,
            return_padding_mask=True,
        )
        self.built = True

    @preprocessing_function
    def call(
        self,
        x,
        y=None,
        sample_weight=None,
        sequence_length=None,
    ):
        sequence_length = sequence_length or self.sequence_length
        x = self.tokenizer(x)
        # Pad with one extra token to account for the truncation below.
        token_ids, padding_mask = self.packer(
            x,
            sequence_length=sequence_length + 1,
            add_start_value=self.add_start_token,
            add_end_value=self.add_end_token,
        )
        # The last token does not have a next token, so we truncate it out.
        x = {
            "token_ids": token_ids[..., :-1],
            "padding_mask": padding_mask[..., :-1],
        }
        # Target `y` will be the next token.
        y, sample_weight = token_ids[..., 1:], padding_mask[..., 1:]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)

    @preprocessing_function
    def generate_preprocess(
        self,
        x,
        sequence_length=None,
    ):
        """Convert strings to integer token input for generation.

        Similar to calling the layer for training, this method takes in strings
        or tensor strings, tokenizes and packs the input, and computes a padding
        mask masking all inputs not filled in with a padded value.

        Unlike calling the layer for training, this method does not compute
        labels and will never append a `tokenizer.end_token_id` to the end of
        the sequence (as generation is expected to continue at the end of the
        inputted prompt).
        """
        if not self.built:
            self.build(None)

        x = self.tokenizer(x)
        token_ids, padding_mask = self.packer(
            x, sequence_length=sequence_length, add_end_value=False
        )
        return {
            "token_ids": token_ids,
            "padding_mask": padding_mask,
        }

    @preprocessing_function
    def generate_postprocess(
        self,
        x,
    ):
        """Convert integer token output to strings for generation.

        This method reverses `generate_preprocess()`, by first removing all
        padding and start/end tokens, and then converting the integer sequence
        back to a string.
        """
        if not self.built:
            self.build(None)

        token_ids, padding_mask = x["token_ids"], x["padding_mask"]
        ids_to_strip = self.tokenizer.special_token_ids
        token_ids = strip_to_ragged(token_ids, padding_mask, ids_to_strip)
        return self.tokenizer.detokenize(token_ids)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "sequence_length": self.sequence_length,
                "add_start_token": self.add_start_token,
                "add_end_token": self.add_end_token,
            }
        )
        return config

    @property
    def sequence_length(self):
        """The padded length of model input sequences."""
        return self._sequence_length

    @sequence_length.setter
    def sequence_length(self, value):
        self._sequence_length = value
        if self.packer is not None:
            self.packer.sequence_length = value

    def export_to_transformers(self, path):
        """Export the preprocessor to HuggingFace Transformers format.

        Args:
            path: str. Path to save the exported preprocessor/tokenizer.
        """
        if self.tokenizer is None:
            raise ValueError("Preprocessor must have a tokenizer for export.")
        from keras_hub.src.utils.transformers.export.hf_exporter import (
            export_tokenizer,
        )

        export_tokenizer(self.tokenizer, path)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.depth_anything.depth_anything_backbone import (
    DepthAnythingBackbone,
)


@keras_hub_export("keras_hub.layers.DepthAnythingImageConverter")
class DepthAnythingImageConverter(ImageConverter):
    backbone_cls = DepthAnythingBackbone



import keras
from keras import activations
from keras import initializers

from keras_hub.src.models.roformer_v2.roformer_v2_attention import (
    RoformerAttention,
)
from keras_hub.src.models.roformer_v2.roformer_v2_attention import RoformerNorm


@keras.saving.register_keras_serializable(package="keras_hub")
class RoformerV2Encoder(keras.layers.Layer):
    """A Transformer Encoder layer for the Roformer backbone."""

    def __init__(
        self,
        heads,
        head_size,
        intermediate_size=None,
        max_wavelength=10000,
        dropout=0,
        activation="relu",
        use_bias=False,
        kernel_initializer="glorot_uniform",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.heads = heads
        self.head_size = head_size
        self.intermediate_size = intermediate_size
        self.use_bias = use_bias
        self.kernel_initializer = initializers.get(kernel_initializer)
        self.max_wavelength = max_wavelength
        self.dropout = dropout
        self.activation = activations.get(activation)

    def build(self, input_shape):
        super().build(input_shape)
        self.attention_layer = RoformerAttention(
            heads=self.heads,
            head_size=self.head_size,
            use_bias=self.use_bias,
            max_wavelength=self.max_wavelength,
            kernel_initializer=self.kernel_initializer,
            dtype=self.dtype_policy,
            name="attention_layer",
        )
        self.attention_layer.build(input_shape)

        self.dropout_layer = keras.layers.Dropout(
            rate=self.dropout,
            dtype=self.dtype_policy,
            name="self_attention_dropout",
        )
        self.dropout_layer.build([])

        # Feedforward layers.
        self.feedforward_intermediate_dense = keras.layers.Dense(
            self.intermediate_size,
            kernel_initializer=self.kernel_initializer,
            use_bias=self.use_bias,
            dtype=self.dtype_policy,
            activation=self.activation,
            name="feedforward_intermediate_dense",
        )
        self.feedforward_intermediate_dense.build(input_shape)

        self.feedforward_output_dense = keras.layers.Dense(
            input_shape[-1],
            kernel_initializer=self.kernel_initializer,
            use_bias=self.use_bias,
            dtype=self.dtype_policy,
            name="feedforward_output_dense",
        )

        self.feedforward_output_dense.build(
            [None, None, self.intermediate_size]
        )

        self.attention_norm = RoformerNorm(
            epsilon=keras.backend.epsilon(),
            name="attention_norm",
            dtype=self.dtype_policy,
        )
        self.attention_norm.build(input_shape)

        self.feedforward_norm = RoformerNorm(
            epsilon=keras.backend.epsilon(),
            name="feedforward_norm",
            dtype=self.dtype_policy,
        )
        self.feedforward_norm.build(input_shape)

    def call(self, x, attention_mask=None):
        attention_output = self.attention_layer(
            x,
            attention_mask=attention_mask,
        )

        residual = x + self.dropout_layer(attention_output)
        x = self.attention_norm(residual)

        intermediate_output = self.feedforward_intermediate_dense(x)
        feedroward_output = self.feedforward_output_dense(intermediate_output)

        residual = x + self.dropout_layer(feedroward_output)
        return self.feedforward_norm(residual)

    def compute_output_shape(self, input_shape):
        return input_shape

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "heads": self.heads,
                "head_size": self.head_size,
                "intermediate_size": self.intermediate_size,
                "max_wavelength": self.max_wavelength,
                "use_bias": self.use_bias,
                "activation": activations.serialize(self.activation),
                "dropout": self.dropout,
                "kernel_initializer": initializers.serialize(
                    self.kernel_initializer
                ),
            }
        )
        return config



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.gemma3.gemma3_backbone import Gemma3Backbone


@keras_hub_export("keras_hub.layers.Gemma3ImageConverter")
class Gemma3ImageConverter(ImageConverter):
    backbone_cls = Gemma3Backbone

    def __init__(self, **kwargs):
        # Always do image preprocessing in float32
        kwargs.pop("dtype", None)
        dtype = "float32"
        super().__init__(dtype=dtype, **kwargs)



"""ResNet preset configurations."""

backbone_presets = {
    "resnet_18_imagenet": {
        "metadata": {
            "description": (
                "18-layer ResNet model pre-trained on the ImageNet 1k dataset "
                "at a 224x224 resolution."
            ),
            "params": 11186112,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv1/keras/resnet_18_imagenet/3",
    },
    "resnet_50_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNet model pre-trained on the ImageNet 1k dataset "
                "at a 224x224 resolution."
            ),
            "params": 23561152,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv1/keras/resnet_50_imagenet/3",
    },
    "resnet_101_imagenet": {
        "metadata": {
            "description": (
                "101-layer ResNet model pre-trained on the ImageNet 1k dataset "
                "at a 224x224 resolution."
            ),
            "params": 42605504,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv1/keras/resnet_101_imagenet/3",
    },
    "resnet_152_imagenet": {
        "metadata": {
            "description": (
                "152-layer ResNet model pre-trained on the ImageNet 1k dataset "
                "at a 224x224 resolution."
            ),
            "params": 58295232,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv1/keras/resnet_152_imagenet/3",
    },
    "resnet_v2_50_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNetV2 model pre-trained on the ImageNet 1k "
                "dataset at a 224x224 resolution."
            ),
            "params": 23561152,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv2/keras/resnet_v2_50_imagenet/3",
    },
    "resnet_v2_101_imagenet": {
        "metadata": {
            "description": (
                "101-layer ResNetV2 model pre-trained on the ImageNet 1k "
                "dataset at a 224x224 resolution."
            ),
            "params": 42605504,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnetv2/keras/resnet_v2_101_imagenet/3",
    },
    "resnet_vd_18_imagenet": {
        "metadata": {
            "description": (
                "18-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 11722824,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_18_imagenet/2",
    },
    "resnet_vd_34_imagenet": {
        "metadata": {
            "description": (
                "34-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 21838408,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_34_imagenet/2",
    },
    "resnet_vd_50_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 25629512,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_50_imagenet/2",
    },
    "resnet_vd_50_ssld_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution with knowledge distillation."
            ),
            "params": 25629512,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_50_ssld_imagenet/2",
    },
    "resnet_vd_50_ssld_v2_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution with knowledge distillation and AutoAugment."
            ),
            "params": 25629512,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_50_ssld_v2_imagenet/2",
    },
    "resnet_vd_50_ssld_v2_fix_imagenet": {
        "metadata": {
            "description": (
                "50-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution with knowledge distillation, AutoAugment and "
                "additional fine-tuning of the classification head."
            ),
            "params": 25629512,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_50_ssld_v2_fix_imagenet/2",
    },
    "resnet_vd_101_imagenet": {
        "metadata": {
            "description": (
                "101-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 44673864,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_101_imagenet/2",
    },
    "resnet_vd_101_ssld_imagenet": {
        "metadata": {
            "description": (
                "101-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution with knowledge distillation."
            ),
            "params": 44673864,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_101_ssld_imagenet/2",
    },
    "resnet_vd_152_imagenet": {
        "metadata": {
            "description": (
                "152-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 60363592,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_152_imagenet/2",
    },
    "resnet_vd_200_imagenet": {
        "metadata": {
            "description": (
                "200-layer ResNetVD (ResNet with bag of tricks) model "
                "pre-trained on the ImageNet 1k dataset at a 224x224 "
                "resolution."
            ),
            "params": 74933064,
            "path": "resnet",
        },
        "kaggle_handle": "kaggle://keras/resnet_vd/keras/resnet_vd_200_imagenet/2",
    },
}



from keras_hub.src.models.basnet.basnet_backbone import BASNetBackbone
from keras_hub.src.models.basnet.basnet_presets import basnet_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(basnet_presets, BASNetBackbone)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.llama.llama_backbone import LlamaBackbone


# LLaMA 3 shares the same architecture as its predecessors
# So, we simply create an alias for API consistency
@keras_hub_export("keras_hub.models.Llama3Backbone")
class Llama3Backbone(LlamaBackbone):
    """
    The Llama Transformer core architecture with hyperparameters.

    This network implements a Transformer-based decoder network,
    Llama, as described in
    ["Llama 7B"](https://arxiv.org/pdf/2310.06825.pdf).
    It includes the embedding lookups and transformer layers.

    The default constructor gives a fully customizable, randomly initialized
    Llama model with any number of layers, heads, and embedding
    dimensions. To load preset architectures and weights, use the `from_preset`
    constructor.

    Args:
        vocabulary_size (int): The size of the token vocabulary.
        num_layers (int): The number of transformer layers.
        num_query_heads (int): The number of query attention heads for
            each transformer.
        hidden_dim (int): The size of the transformer encoding and pooling
            layers.
        intermediate_dim (int): The output dimension of the first Dense layer in
            a three-layer feedforward network for each transformer.
        num_key_value_heads (int): The number of key and value attention heads
            fo each transformer.
        rope_max_wavelength (int, optional): The maximum angular wavelength of
            the sine/cosine curves, for rotary embeddings. Defaults to `10000`.
        rope_position_scaling_factor (float, optional): The scaling factor for
            calculation of roatary embedding. Defaults to `1.0`
        rope_requency_adjustment_factor (float, optional): The scaling factor
            used to scale the inverse frequencies.
        rope_low_freq_factor (float, optional): The low frequency factor.
            Defaults to None.
        rope_high_freq_factor: (float, optional) Used for Llama3.1+. The high
            frequency factor. Defaults to None.
        rope_pretraining_sequence_length: (int, optional) Sequence length during
            original pretraining. Defaults to None.
        layer_norm_epsilon (float, optional): Epsilon for the layer
            normalization layers in the transformer decoder. Defaults to `1e-6`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for model computations and weights. Note that some computations,
            such as softmax and layer normalization, will always be done at
            float32 precision regardless of dtype.

    Examples:

    ```python
    input_data = {
        "token_ids": np.ones(shape=(1, 12), dtype="int32"),
        "padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]]),
    }

    # Pretrained Llama decoder.
    model = keras_hub.models.Llama3Backbone.from_preset("llama3_8b_en")
    model(input_data)

    # Randomly initialized Llama decoder with custom config.
    model = keras_hub.models.Llama3Backbone(
        vocabulary_size=10,
        hidden_dim=512,
        num_layers=2,
        num_query_heads=32,
        num_key_value_heads=8,
        intermediate_dim=1024,
        layer_norm_epsilon=1e-6,
        dtype="float32"
    )
    model(input_data)
    ```
    """

    pass



# Metadata for loading pretrained model weights.
backbone_presets = {
    "hgnetv2_b4_ssld_stage2_ft_in1k": {
        "metadata": {
            "description": (
                "HGNetV2 B4 model with 2-stage SSLD training, fine-tuned on "
                "ImageNet-1K."
            ),
            "params": 13599072,
            "path": "hgnetv2",
        },
        "kaggle_handle": "kaggle://keras/hgnetv2/keras/hgnetv2_b4_ssld_stage2_ft_in1k/1",
    },
    "hgnetv2_b5_ssld_stage1_in22k_in1k": {
        "metadata": {
            "description": (
                "HGNetV2 B5 model with 1-stage SSLD training, pre-trained on "
                "ImageNet-22K and fine-tuned on ImageNet-1K."
            ),
            "params": 33419680,
            "path": "hgnetv2",
        },
        "kaggle_handle": "kaggle://keras/hgnetv2/keras/hgnetv2_b5_ssld_stage1_in22k_in1k/1",
    },
    "hgnetv2_b5_ssld_stage2_ft_in1k": {
        "metadata": {
            "description": (
                "HGNetV2 B5 model with 2-stage SSLD training, fine-tuned on "
                "ImageNet-1K."
            ),
            "params": 33419680,
            "path": "hgnetv2",
        },
        "kaggle_handle": "kaggle://keras/hgnetv2/keras/hgnetv2_b5_ssld_stage2_ft_in1k/1",
    },
    "hgnetv2_b6_ssld_stage1_in22k_in1k": {
        "metadata": {
            "description": (
                "HGNetV2 B6 model with 1-stage SSLD training, pre-trained on "
                "ImageNet-22K and fine-tuned on ImageNet-1K."
            ),
            "params": 69179888,
            "path": "hgnetv2",
        },
        "kaggle_handle": "kaggle://keras/hgnetv2/keras/hgnetv2_b6_ssld_stage1_in22k_in1k/1",
    },
    "hgnetv2_b6_ssld_stage2_ft_in1k": {
        "metadata": {
            "description": (
                "HGNetV2 B6 model with 2-stage SSLD training, fine-tuned on "
                "ImageNet-1K."
            ),
            "params": 69179888,
            "path": "hgnetv2",
        },
        "kaggle_handle": "kaggle://keras/hgnetv2/keras/hgnetv2_b6_ssld_stage2_ft_in1k/1",
    },
}



from keras import layers

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.token_and_position_embedding import (
    TokenAndPositionEmbedding,
)
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.clip.clip_layers import CLIPEncoderLayer


@keras_hub_export("keras_hub.models.CLIPTextEncoder")
class CLIPTextEncoder(Backbone):
    """CLIP text core network with hyperparameters.

    Args:
        vocabulary_size: int. The size of the token vocabulary.
        embedding_dim: int. The output dimension of the embedding layer.
        hidden_dim: int. The size of the transformer hidden state at the end
            of each transformer layer.
        num_layers: int. The number of transformer layers.
        num_heads: int. The number of attention heads for each transformer.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer.
        intermediate_activation: activation function. The activation that
            is used for the first Dense layer in a two-layer feedforward network
            for each transformer.
        intermediate_output_index: optional int. The index of the intermediate
            output. If specified, the output will become a dictionary with two
            keys `"sequence_output"` and `"intermediate_output"`.
        max_sequence_length: int. The maximum sequence length that this encoder
            can consume.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for the models computations and weights. Note that some
            computations, such as softmax and layer normalization will always
            be done a float32 precision regardless of dtype.
    """

    def __init__(
        self,
        vocabulary_size,
        embedding_dim,
        hidden_dim,
        num_layers,
        num_heads,
        intermediate_dim,
        intermediate_activation="quick_gelu",
        intermediate_output_index=None,
        max_sequence_length=77,
        dtype=None,
        name=None,
        **kwargs,
    ):
        if (
            intermediate_output_index is not None
            and intermediate_output_index < 0
        ):
            intermediate_output_index += num_layers

        # `prefix` is used to prevent duplicate name when utilizing multiple
        # CLIP models within a single model, such as in StableDiffusion3.
        prefix = str(name) + "_" if name is not None else ""

        # === Layers ===
        self.embedding = TokenAndPositionEmbedding(
            vocabulary_size=vocabulary_size,
            sequence_length=max_sequence_length,
            embedding_dim=embedding_dim,
            dtype=dtype,
            name=f"{prefix}embedding",
        )
        self.encoder_layers = [
            CLIPEncoderLayer(
                hidden_dim,
                num_heads,
                intermediate_dim,
                intermediate_activation,
                dtype=dtype,
                name=f"{prefix}encoder_block_{i}",
            )
            for i in range(num_layers)
        ]
        self.layer_norm = layers.LayerNormalization(
            epsilon=1e-6, dtype=dtype, name=f"{prefix}layer_norm"
        )

        # === Functional Model ===
        token_id_input = layers.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        x = self.embedding(token_id_input)
        intermediate_output = None
        for i, block in enumerate(self.encoder_layers):
            x = block(x)
            if i == intermediate_output_index:
                intermediate_output = x
        x = self.layer_norm(x)
        sequence_output = x

        if intermediate_output_index is not None:
            outputs = {
                "sequence_output": sequence_output,
                "intermediate_output": intermediate_output,
            }
        else:
            outputs = sequence_output
        super().__init__(
            inputs={"token_ids": token_id_input},
            outputs=outputs,
            dtype=dtype,
            name=name,
            **kwargs,
        )

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.max_sequence_length = max_sequence_length
        self.embedding_dim = embedding_dim
        self.hidden_dim = hidden_dim
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.intermediate_dim = intermediate_dim
        self.intermediate_activation = intermediate_activation
        self.intermediate_output_index = intermediate_output_index

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vocabulary_size": self.vocabulary_size,
                "embedding_dim": self.embedding_dim,
                "hidden_dim": self.hidden_dim,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "intermediate_dim": self.intermediate_dim,
                "intermediate_activation": self.intermediate_activation,
                "intermediate_output_index": self.intermediate_output_index,
                "max_sequence_length": self.max_sequence_length,
            }
        )
        return config



from keras_hub.src.models.t5gemma.t5gemma_backbone import T5GemmaBackbone
from keras_hub.src.models.t5gemma.t5gemma_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, T5GemmaBackbone)



import numpy as np

from keras_hub.src.models.qwen3_moe.qwen3_moe_backbone import Qwen3MoeBackbone
from keras_hub.src.utils.preset_utils import load_json

backbone_cls = Qwen3MoeBackbone


def convert_backbone_config(transformers_config):
    return {
        "vocabulary_size": transformers_config["vocab_size"],
        "hidden_dim": transformers_config["hidden_size"],
        "head_dim": transformers_config["head_dim"],
        "num_layers": transformers_config["num_hidden_layers"],
        "num_query_heads": transformers_config["num_attention_heads"],
        "num_key_value_heads": transformers_config["num_key_value_heads"],
        "intermediate_dim": transformers_config["intermediate_size"],
        "moe_intermediate_dim": transformers_config["moe_intermediate_size"],
        "num_experts": transformers_config["num_experts"],
        "top_k": transformers_config["num_experts_per_tok"],
        "norm_top_k_prob": transformers_config["norm_topk_prob"],
        "decoder_sparse_step": transformers_config["decoder_sparse_step"],
        "layer_norm_epsilon": transformers_config["rms_norm_eps"],
        "rope_max_wavelength": transformers_config["rope_theta"],
        "sliding_window_size": transformers_config["sliding_window"],
        "router_aux_loss_coefficient": transformers_config[
            "router_aux_loss_coef"
        ],
        "tie_word_embeddings": transformers_config.get(
            "tie_word_embeddings", False
        ),
    }


def convert_weights(backbone, loader, transformers_config):
    loader.port_weight(
        keras_variable=backbone.get_layer("token_embedding").embeddings,
        hf_weight_key="model.embed_tokens.weight",
    )
    if not backbone.tie_word_embeddings:
        loader.port_weight(
            keras_variable=backbone.get_layer(
                "token_embedding"
            ).reverse_embeddings,
            hf_weight_key="lm_head.weight",
            # rearrange_pattern="b a -> a b",
            hook_fn=lambda hf_tensor, _: np.transpose(hf_tensor, axes=(1, 0)),
        )

    def transpose_and_reshape(x, shape):
        return np.reshape(np.transpose(x), shape)

    for i in range(backbone.num_layers):
        decoder_layer = backbone.get_layer(f"transformer_layer_{i}")

        # Input layernorm
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layernorm.scale,
            hf_weight_key=f"model.layers.{i}.input_layernorm.weight",
        )

        # Attention layers

        ## Query
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._query_dense.kernel,
            hf_weight_key=f"model.layers.{i}.self_attn.q_proj.weight",
            hook_fn=transpose_and_reshape,
        )
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._query_dense_layer_norm.scale,
            hf_weight_key=f"model.layers.{i}.self_attn.q_norm.weight",
        )
        ## Key
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._key_dense.kernel,
            hf_weight_key=f"model.layers.{i}.self_attn.k_proj.weight",
            hook_fn=transpose_and_reshape,
        )
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._key_dense_layer_norm.scale,
            hf_weight_key=f"model.layers.{i}.self_attn.k_norm.weight",
        )
        ## Value
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._value_dense.kernel,
            hf_weight_key=f"model.layers.{i}.self_attn.v_proj.weight",
            hook_fn=transpose_and_reshape,
        )
        ## Output
        loader.port_weight(
            keras_variable=decoder_layer._self_attention_layer._output_dense.kernel,
            hf_weight_key=f"model.layers.{i}.self_attn.o_proj.weight",
            # rearrange_patterns="c (a b) -> a b c",
            # rearrange_dims={"a": backbone.num_query_heads},
            hook_fn=transpose_and_reshape,
        )

        # MLP layers
        if (
            (i not in backbone.mlp_only_layers)
            and backbone.num_experts > 0
            and ((i + 1) % backbone.decoder_sparse_step == 0)
        ):
            # MoE layers
            loader.port_weight(
                keras_variable=decoder_layer.mlp._sparse_feedforward_gate_dense.kernel,
                hf_weight_key=f"model.layers.{i}.mlp.gate.weight",
                # rearrange_patterns="b a -> a b",
                hook_fn=lambda hf_tensor, _: np.transpose(
                    hf_tensor, axes=(1, 0)
                ),
            )
            # Batched experts: gate_up_proj and down_proj
            gate_up_proj_list = []
            down_proj_list = []
            for expert_idx in range(backbone.num_experts):
                # Load gate_proj and up_proj for each expert
                gate_proj = loader.get_tensor(
                    f"model.layers.{i}.mlp.experts.{expert_idx}.gate_proj.weight"
                )
                up_proj = loader.get_tensor(
                    f"model.layers.{i}.mlp.experts.{expert_idx}.up_proj.weight"
                )
                # Transpose to (hidden_dim, intermediate_dim)
                gate_proj = np.transpose(gate_proj, axes=(1, 0))
                up_proj = np.transpose(up_proj, axes=(1, 0))
                # Concatenate gate_proj and up_proj along the last dimension
                gate_up_proj = np.concatenate([gate_proj, up_proj], axis=-1)
                gate_up_proj_list.append(gate_up_proj)

                # Load down_proj for each expert
                down_proj = loader.get_tensor(
                    f"model.layers.{i}.mlp.experts.{expert_idx}.down_proj.weight"
                )
                down_proj = np.transpose(
                    down_proj, axes=(1, 0)
                )  # (intermediate_dim, hidden_dim)
                down_proj_list.append(down_proj)

            # Stack the lists to create batched weights
            gate_up_proj_batched = np.stack(
                gate_up_proj_list, axis=0
            )  # (num_experts, hidden_dim, 2 * intermediate_dim)
            down_proj_batched = np.stack(
                down_proj_list, axis=0
            )  # (num_experts, intermediate_dim, hidden_dim)

            # Assign batched weights to expert_bank
            decoder_layer.mlp.expert_bank._expert_feedforward_gate_dense.assign(
                gate_up_proj_batched
            )
            decoder_layer.mlp.expert_bank._expert_feedforward_output_dense.assign(
                down_proj_batched
            )
        else:
            loader.port_weight(
                keras_variable=decoder_layer._feedforward_intermediate_dense.kernel,
                hf_weight_key=f"model.layers.{i}.mlp.up_proj.weight",
                # rearrange_patterns="b a -> a b",
                hook_fn=lambda hf_tensor, _: np.transpose(
                    hf_tensor, axes=(1, 0)
                ),
            )
            loader.port_weight(
                keras_variable=decoder_layer._feedforward_output_dense.kernel,
                hf_weight_key=f"model.layers.{i}.mlp.down_proj.weight",
                # rearrange_patterns="b a -> a b",
                hook_fn=lambda hf_tensor, _: np.transpose(
                    hf_tensor, axes=(1, 0)
                ),
            )
            loader.port_weight(
                keras_variable=decoder_layer._feedforward_gate_dense.kernel,
                hf_weight_key=f"model.layers.{i}.mlp.gate_proj.weight",
                # rearrange_patterns="b a -> a b",
                hook_fn=lambda hf_tensor, _: np.transpose(
                    hf_tensor, axes=(1, 0)
                ),
            )

        # Feedforward layernorm
        loader.port_weight(
            keras_variable=decoder_layer._feedforward_layernorm.scale,
            hf_weight_key=f"model.layers.{i}.post_attention_layernorm.weight",
        )

    # Final normalization layer
    loader.port_weight(
        keras_variable=backbone.get_layer("sequence_output_layernorm").scale,
        hf_weight_key="model.norm.weight",
    )

    return backbone


def convert_tokenizer(cls, preset, **kwargs):
    tokenizer_config = load_json(preset, "tokenizer.json")
    vocab = tokenizer_config["model"]["vocab"]
    merges = tokenizer_config["model"]["merges"]
    merges = [" ".join(item) for item in merges]

    # Load all special tokens with the exception of "reserved" ones.
    special_tokens = set()
    for token in tokenizer_config["added_tokens"]:
        if not token["content"].startswith("<|reserved_special_token_"):
            vocab[token["content"]] = token["id"]
            special_tokens.add(token["content"])

    kwargs.update(
        {
            "unsplittable_tokens": list(special_tokens),
        }
    )

    return cls(vocabulary=vocab, merges=merges, **kwargs)



from keras_hub.src.models.mobilenetv5.mobilenetv5_backbone import (
    MobileNetV5Backbone,
)
from keras_hub.src.models.mobilenetv5.mobilenetv5_presets import (
    backbone_presets,
)
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, MobileNetV5Backbone)



import keras
from keras import activations

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.reversible_embedding import (
    ReversibleEmbedding,
)
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.roformer_v2.roformer_v2_attention import RoformerNorm
from keras_hub.src.models.roformer_v2.roformer_v2_encoder import (
    RoformerV2Encoder,
)


def roformer_kernel_initializer(stddev=0.02):
    return keras.initializers.TruncatedNormal(stddev=stddev)


@keras_hub_export("keras_hub.models.RoformerV2Backbone")
class RoformerV2Backbone(Backbone):
    """A RoformerV2 encoder network.

    This class implements a bi-directional Transformer-based encoder as
    described in ["Roformer"](https://github.com/ZhuiyiTechnology/roformer).
    It includes the
    embedding lookups and transformer layers, but not the masked language model
    or next sentence prediction heads.

    The default constructor gives a fully customizable, randomly initialized
    RoformerV2 encoder with any number of layers, heads, and embed dim.To
    load preset architectures and weights, use the `from_preset()` constructor.

    Disclaimer: Pre-trained models are provided on an "as is" basis, without
    warranties or conditions of any kind.

    Args:
        vocabulary_size: int. The size of the token vocabulary.
        num_layers: int. The number of transformer layers.
        num_heads: int. The number of attention heads for each transformer.
            The hidden size must be divisible by the number of attention heads.
        hidden_dim: int. The size of the transformer encoding and pooler layers.
        intermediate_dim: int. The output dimension of the first Dense layer in
            a two-layer feedforward network for each transformer.
        dropout: float. Dropout probability for the Transformer encoder.
        num_segments: int. The number of types that the 'segment_ids' input can
            take.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for model computations and weights. Note that some computations,
            such as softmax and layer normalization, will always be done at
            float32 precision regardless of dtype.

    Examples:
    ```python
    input_data = {
        "token_ids": np.ones(shape=(1, 12), dtype="int32"),
        "segment_ids": np.array([[0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0]]),
        "padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]]),
    }

    # Pretrained RoformerV2 encoder.
    model = keras_hub.models.RoformerV2Backbone.from_preset("roformer_v2_base")
    model(input_data)

    # Randomly initialized RoformerV2 encoder with a custom config.
    model = keras_hub.models.RoformerV2Backbone(
        vocabulary_size=30552,
        num_layers=4,
        num_heads=4,
        hidden_dim=256,
        intermediate_dim=512,
        head_size = 64,
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        vocabulary_size,
        num_layers,
        num_heads,
        hidden_dim,
        intermediate_dim,
        head_size,
        use_bias=False,
        activation="relu",
        dropout=0.1,
        num_segments=2,
        dtype=None,
        max_wavelength=10000,
        **kwargs,
    ):
        # === Layers ===
        self.token_embedding = ReversibleEmbedding(
            input_dim=vocabulary_size,
            output_dim=hidden_dim,
            embeddings_initializer=roformer_kernel_initializer(),
            dtype=dtype,
            name="token_embedding",
        )
        self.segment_embedding = keras.layers.Embedding(
            input_dim=num_segments,
            output_dim=hidden_dim,
            embeddings_initializer=roformer_kernel_initializer(),
            dtype=dtype,
            name="segment_embedding",
        )
        self.embeddings_add = keras.layers.Add(
            dtype=dtype,
            name="embeddings_add",
        )
        self.embeddings_layer_norm = RoformerNorm(
            epsilon=keras.backend.epsilon(),
            dtype=dtype,
            name="embeddings_layer_norm",
        )
        self.embeddings_dropout = keras.layers.Dropout(
            dropout,
            dtype=dtype,
            name="embeddings_dropout",
        )
        self.transformer_layers = []
        for i in range(num_layers):
            layer = RoformerV2Encoder(
                heads=num_heads,
                head_size=head_size,
                intermediate_size=intermediate_dim,
                use_bias=use_bias,
                max_wavelength=max_wavelength,
                dropout=dropout,
                activation=activation,
                kernel_initializer=roformer_kernel_initializer(),
                dtype=dtype,
                name=f"transformer_layer_{i}",
            )
            self.transformer_layers.append(layer)

        # === Functional Model ===
        token_id_input = keras.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        segment_id_input = keras.Input(
            shape=(None,), dtype="int32", name="segment_ids"
        )
        attention_mask = keras.ops.not_equal(token_id_input, 0)
        # Embed tokens, positions, and segment ids.
        tokens = self.token_embedding(token_id_input)
        segments = self.segment_embedding(segment_id_input)
        # Sum, normalize and apply dropout to embeddings.
        x = self.embeddings_add((tokens, segments))
        x = self.embeddings_layer_norm(x)
        x = self.embeddings_dropout(x)
        for transformer_layer in self.transformer_layers:
            x = transformer_layer(x, attention_mask=attention_mask)

        super().__init__(
            inputs={
                "token_ids": token_id_input,
                "segment_ids": segment_id_input,
            },
            outputs=x,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.dropout = dropout
        self.num_segments = num_segments
        self.max_wavelength = max_wavelength
        self.head_size = head_size
        self.dropout = dropout
        self.activation = activations.get(activation)
        self.use_bias = use_bias
        self.start_token_index = 0

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vocabulary_size": self.vocabulary_size,
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "dropout": self.dropout,
                "num_segments": self.num_segments,
                "max_wavelength": self.max_wavelength,
                "head_size": self.head_size,
                "use_bias": self.use_bias,
                "activation": activations.serialize(self.activation),
            }
        )
        return config



"""MobileNet preset configurations."""

backbone_presets = {
    "mobilenet_v3_small_050_imagenet": {
        "metadata": {
            "description": (
                "Small Mobilenet V3 model pre-trained on the ImageNet 1k "
                "dataset at a 224x224 resolution. Has half channel multiplier."
            ),
            "params": 278784,
            "path": "mobilenet",
        },
        "kaggle_handle": "kaggle://keras/mobilenetv3/keras/mobilenet_v3_small_050_imagenet/1",
    },
    "mobilenet_v3_small_100_imagenet": {
        "metadata": {
            "description": (
                "Small Mobilenet V3 model pre-trained on the ImageNet 1k "
                "dataset at a 224x224 resolution. Has baseline channel "
                "multiplier."
            ),
            "params": 939120,
            "path": "mobilenet",
        },
        "kaggle_handle": "kaggle://keras/mobilenetv3/keras/mobilenet_v3_small_100_imagenet/1",
    },
    "mobilenet_v3_large_100_imagenet": {
        "metadata": {
            "description": (
                "Large Mobilenet V3 model pre-trained on the ImageNet 1k "
                "dataset at a 224x224 resolution. Has baseline channel "
                "multiplier."
            ),
            "params": 2996352,
            "path": "mobilenet",
        },
        "kaggle_handle": "kaggle://keras/mobilenetv3/keras/mobilenet_v3_large_100_imagenet/1",
    },
    "mobilenet_v3_large_100_imagenet_21k": {
        "metadata": {
            "description": (
                "Large Mobilenet V3 model pre-trained on the ImageNet 21k "
                "dataset at a 224x224 resolution. Has baseline channel "
                "multiplier."
            ),
            "params": 2996352,
            "path": "mobilenet",
        },
        "kaggle_handle": "kaggle://keras/mobilenetv3/keras/mobilenet_v3_large_100_imagenet_21k/1",
    },
}



"""DepthAnything model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "depth_anything_v2_small": {
        "metadata": {
            "description": (
                "Small variant of Depth Anything V2 monocular depth estimation "
                "(MDE) model trained on synthetic labeled images and real "
                "unlabeled images."
            ),
            "params": 25_311_169,
            "path": "depth_anything",
        },
        "kaggle_handle": "kaggle://keras/depth-anything/keras/depth_anything_v2_small/1",
    },
    "depth_anything_v2_base": {
        "metadata": {
            "description": (
                "Base variant of Depth Anything V2 monocular depth estimation "
                "(MDE) model trained on synthetic labeled images and real "
                "unlabeled images."
            ),
            "params": 98_522_945,
            "path": "depth_anything",
        },
        "kaggle_handle": "kaggle://keras/depth-anything/keras/depth_anything_v2_base/1",
    },
    "depth_anything_v2_large": {
        "metadata": {
            "description": (
                "Large variant of Depth Anything V2 monocular depth estimation "
                "(MDE) model trained on synthetic labeled images and real "
                "unlabeled images."
            ),
            "params": 336_718_529,
            "path": "depth_anything",
        },
        "kaggle_handle": "kaggle://keras/depth-anything/keras/depth_anything_v2_large/1",
    },
}



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_classifier import ImageClassifier
from keras_hub.src.models.xception.xception_backbone import XceptionBackbone
from keras_hub.src.models.xception.xception_image_classifier_preprocessor import (  # noqa: E501
    XceptionImageClassifierPreprocessor,
)


@keras_hub_export("keras_hub.models.XceptionImageClassifier")
class XceptionImageClassifier(ImageClassifier):
    backbone_cls = XceptionBackbone
    preprocessor_cls = XceptionImageClassifierPreprocessor



import math

from keras import activations
from keras import initializers
from keras import layers
from keras import ops

from keras_hub.src.layers.modeling.transformer_layer_utils import (
    compute_causal_mask,
)
from keras_hub.src.layers.modeling.transformer_layer_utils import (
    merge_padding_and_attention_mask,
)
from keras_hub.src.models.smollm3.smollm3_utils import apply_rotary_pos_emb
from keras_hub.src.models.smollm3.smollm3_utils import rope_init


class SmolLM3Attention(layers.Layer):
    """Multi-head attention layer for SmolLM3 model.

    Args:
        hidden_size: int. The hidden size of the attention layer.
        num_attention_heads: int. The number of attention heads.
        num_key_value_heads: int. The number of key-value heads.
        attention_bias: bool. Whether to use bias in attention projections.
        attention_dropout: float. Dropout rate for attention weights.
        rope_layer_enabled_list: list of bool. List indicating if RoPE is
            enabled for each layer.
        layer_types: list of str. List of layer types.
        layer_idx: int. Index of the current layer.
        max_position_embeddings: int. Maximum sequence length for position
            embeddings. Defaults to 2048.
        rope_theta: float. The theta value for RoPE. Defaults to 10000.0.
        partial_rotary_factor: float. The factor for partial rotary embedding.
            Defaults to 1.0.
    """

    def __init__(
        self,
        hidden_size,
        num_attention_heads,
        num_key_value_heads,
        attention_bias,
        attention_dropout,
        rope_layer_enabled_list,
        layer_types,
        layer_idx,
        max_position_embeddings=2048,
        rope_theta=10000.0,
        partial_rotary_factor=1.0,
        **kwargs,
    ):
        super().__init__(**kwargs)

        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.num_key_value_heads = num_key_value_heads
        self.attention_bias = attention_bias
        self.attention_dropout = attention_dropout
        self.rope_layer_enabled_list = rope_layer_enabled_list
        self.layer_types = layer_types
        self.max_position_embeddings = max_position_embeddings
        self.rope_theta = rope_theta
        self.partial_rotary_factor = partial_rotary_factor

        self._dot_product_equation = "bquh,bkuh->buqk"
        self._combine_equation = "buqk,bkuh->bquh"

        self.head_dim = hidden_size // self.num_attention_heads
        self._inv_norm_factor = 1.0 / math.sqrt(self.head_dim)

        self.layer_idx = layer_idx
        self.num_key_value_groups = (
            self.num_attention_heads // self.num_key_value_heads
        )
        self.scaling = self.head_dim**-0.5
        self.is_causal = True

        self.q_proj = layers.Dense(
            self.num_attention_heads * self.head_dim,
            use_bias=self.attention_bias,
            name="q_proj",
        )
        self.k_proj = layers.Dense(
            self.num_key_value_heads * self.head_dim,
            use_bias=self.attention_bias,
            name="k_proj",
        )
        self.v_proj = layers.Dense(
            self.num_key_value_heads * self.head_dim,
            use_bias=self.attention_bias,
            name="v_proj",
        )
        self.o_proj = layers.EinsumDense(
            equation="bquh,uhm->bqm",
            output_shape=(None, self.hidden_size),
            name="o_proj",
        )
        self.o_proj.build((None, None, self.num_attention_heads, self.head_dim))

        self.use_rope = (
            self.rope_layer_enabled_list[self.layer_idx]
            if self.layer_idx < len(self.rope_layer_enabled_list)
            else True
        )  # Default to True if index out of bounds

        self.rotary_embedding = SmolLM3RotaryEmbedding(
            hidden_size=self.hidden_size,
            num_attention_heads=self.num_attention_heads,
            max_position_embeddings=self.max_position_embeddings,
            rope_theta=self.rope_theta,
            partial_rotary_factor=self.partial_rotary_factor,
            name="rotary_emb",
        )

        self._softmax = layers.Softmax(
            axis=-1,
            dtype="float32",
            name="attention_softmax",
        )

    def build(self, input_shape):
        """Builds the internal Dense layers.

        Args:
            input_shape: A list/tuple of shapes for the inputs:
                [hidden_states_shape, position_embeddings_shape_tuple,
                attention_mask_shape]
                - hidden_states_shape: (batch_size, seq_len,
                    hidden_size)
        """
        # The input shape to the Dense layers (q_proj, k_proj, v_proj, o_proj)
        # is the same as the hidden_states input to SmolLM3Attention.
        hidden_states_shape = input_shape[0]
        self.q_proj.build(hidden_states_shape)
        self.k_proj.build(hidden_states_shape)
        self.v_proj.build(hidden_states_shape)
        super().build(input_shape)

    def call(
        self,
        hidden_states,
        training=False,
        attention_mask=None,
        **kwargs,
    ):
        """Forward pass for SmolLM3Attention.

        Args:
            hidden_states: Input tensor of shape (batch_size, seq_len,
                hidden_size).
            position_embeddings: Tuple of (cos, sin) tensors for RoPE.
            attention_mask: Attention mask tensor.
            training: Whether the layer is in training mode.
        """
        self.training = training
        self_attention_cache = kwargs.get("self_attention_cache", None)
        self_attention_cache_update_index = kwargs.get(
            "self_attention_cache_update_index", None
        )
        start_index = (
            self_attention_cache_update_index
            if self_attention_cache_update_index is not None
            else 0
        )

        input_shape = ops.shape(hidden_states)[:-1]
        hidden_shape = (*input_shape, self.num_attention_heads, self.head_dim)

        query = ops.reshape(self.q_proj(hidden_states), hidden_shape)

        def _compute_kv_values(x_input):
            kv_hidden_shape = (
                *input_shape,
                self.num_key_value_heads,
                self.head_dim,
            )

            key = ops.reshape(self.k_proj(x_input), kv_hidden_shape)
            value = ops.reshape(self.v_proj(x_input), kv_hidden_shape)

            return key, value

        if self_attention_cache is not None:
            key_cache = self_attention_cache[:, 0, ...]
            value_cache = self_attention_cache[:, 1, ...]

            if self_attention_cache_update_index is None:
                key = key_cache
                value = value_cache
            else:
                key_update, value_update = _compute_kv_values(hidden_states)

                # Apply RoPE to key_update BEFORE caching
                if self.use_rope:
                    cos, sin = self.rotary_embedding(
                        query, start_index=start_index
                    )
                    query_rope, key_update = apply_rotary_pos_emb(
                        query, key_update, cos, sin, expansion_axis=2
                    )
                    query = query_rope

                start = (0, self_attention_cache_update_index, 0, 0)

                key = ops.slice_update(key_cache, start, key_update)
                value = ops.slice_update(value_cache, start, value_update)
                self_attention_cache = ops.stack((key, value), axis=1)
        else:
            if self_attention_cache_update_index is not None:
                raise ValueError(
                    "`self_attention_cache_update_index` should not be set "
                    "if `self_attention_cache` is `None`. Received: "
                    f"self_attention_cache={self_attention_cache}, "
                    "self_attention_cache_update_index="
                    f"{self_attention_cache_update_index}"
                )
            key, value = _compute_kv_values(hidden_states)

            # Apply RoPE when not using cache
            if self.use_rope:
                cos, sin = self.rotary_embedding(query, start_index=start_index)
                query, key = apply_rotary_pos_emb(
                    query, key, cos, sin, expansion_axis=2
                )

        key = ops.repeat(key, repeats=self.num_key_value_groups, axis=2)
        value = ops.repeat(value, repeats=self.num_key_value_groups, axis=2)

        attn_output = self._compute_attention(
            query,
            key,
            value,
            attention_mask,
            cache_update_index=self_attention_cache_update_index,
        )

        attn_output = self.o_proj(attn_output)

        if self_attention_cache is not None:
            return attn_output, self_attention_cache

        return attn_output

    def compute_output_shape(self, input_shape):
        """
        Computes the output shape of the layer.

        Args:
            input_shape: A list/tuple of shapes for the inputs:
                [hidden_states_shape, position_embeddings_shape_tuple,
                attention_mask_shape]
                - hidden_states_shape: (batch_size, seq_len,
                  hidden_size)
                - position_embeddings_shape_tuple: (cos_shape,
                  sin_shape) where cos_shape/sin_shape is
                  (batch_size, seq_len, head_dim)
                - attention_mask_shape: (batch_size, 1, seq_len,
                  seq_len)

        Returns:
            A list of output shapes: [output_attn_output_shape,
            output_attn_weights_shape]
        """
        hidden_states_shape = input_shape[0]

        batch_size = hidden_states_shape[0]
        seq_len = hidden_states_shape[1]

        output_attn_output_shape = (batch_size, seq_len, self.hidden_size)

        output_attn_weights_shape = (
            batch_size,
            self.num_attention_heads,
            seq_len,
            seq_len,
        )

        return [output_attn_output_shape, output_attn_weights_shape]

    def _masked_softmax(self, attention_scores, attention_mask=None):
        """Applies softmax with optional masking.

        Args:
            attention_scores: Attention score tensor.
            attention_mask: Optional mask tensor.

        Returns:
            Masked softmax attention weights.
        """
        if attention_mask is not None:
            return self._softmax(
                attention_scores, attention_mask[:, None, :, :]
            )
        return self._softmax(attention_scores)

    def _compute_attention(
        self, query, key, value, attention_mask=None, cache_update_index=None
    ):
        """Computes attention using query, key, and value tensors.

        Uses Flash Attention when available for better performance.

        Args:
            query: Query tensor.
            key: Key tensor.
            value: Value tensor.
            attention_mask: Optional mask tensor.
            cache_update_index: Index for sliding window computation.

        Returns:
            attention_output: Output tensor after applying attention.
        """
        attention_scores = ops.einsum(self._dot_product_equation, query, key)

        attention_scores = ops.multiply(
            attention_scores,
            ops.cast(self._inv_norm_factor, self.compute_dtype),
        )
        attention_scores = self._masked_softmax(
            attention_scores, attention_mask
        )
        attention_scores = ops.cast(attention_scores, self.compute_dtype)
        attention_output = ops.einsum(
            self._combine_equation, attention_scores, value
        )

        return attention_output


class SmolLM3MLP(layers.Layer):
    """Multi-layer perceptron (MLP) block for SmolLM3 model.

    Args:
        hidden_size: int. The hidden size of the MLP.
        intermediate_size: int. The intermediate size of the MLP.
        mlp_bias: bool. Whether to use bias in MLP dense layers.
    """

    def __init__(self, hidden_size, intermediate_size, mlp_bias, **kwargs):
        super().__init__(**kwargs)
        self.hidden_size = hidden_size
        self.intermediate_size = intermediate_size
        self.mlp_bias = mlp_bias

        self.gate_proj = layers.Dense(
            self.intermediate_size, use_bias=self.mlp_bias, name="gate_proj"
        )
        self.up_proj = layers.Dense(
            self.intermediate_size, use_bias=self.mlp_bias, name="up_proj"
        )
        self.down_proj = layers.Dense(
            self.hidden_size, use_bias=self.mlp_bias, name="down_proj"
        )

    def build(self, input_shape):
        """
        Builds the internal Dense layers.
        Args:
            input_shape: The shape of the input to this layer
                         (batch_size, seq_len, hidden_size).
        """
        self.gate_proj.build(input_shape)
        self.up_proj.build(input_shape)
        # The down_proj takes intermediate_output, which has shape
        # (batch_size, seq_len, intermediate_size)
        down_proj_input_shape = (
            input_shape[0],
            input_shape[1],
            self.intermediate_size,
        )
        self.down_proj.build(down_proj_input_shape)
        super().build(input_shape)

    def call(self, x):
        """
        Forward pass for SmolLM3MLP.

        Args:
            x: Input tensor of shape (batch_size, seq_len, hidden_size).
        """
        gate_output = activations.silu(self.gate_proj(x))
        up_output = self.up_proj(x)
        intermediate_output = gate_output * up_output
        down_proj_output = self.down_proj(intermediate_output)
        return down_proj_output

    def compute_output_shape(self, input_shape):
        """
        Computes the output shape of the layer.

        Args:
            input_shape: The input shape (batch_size, seq_len, hidden_size).

        Returns:
            The output shape, which is the same as the input shape:
            (batch_size, seq_len, hidden_size).
        """
        return input_shape


class SmolLM3DecoderLayer(layers.Layer):
    """Decoder layer for SmolLM3 model, combining self-attention and MLP.

    Args:
        hidden_size: int. The hidden size of the layer.
        num_attention_heads: int. The number of attention heads.
        num_key_value_heads: int. The number of key-value heads.
        attention_bias: bool. Whether to use bias in attention projections.
        attention_dropout: float. Dropout rate for attention weights.
        rope_layer_enabled_list: list of bool. List indicating if RoPE is
            enabled for each layer.
        layer_types: list of str. List of layer types.
        layer_idx: int. Index of the current layer.
        intermediate_size: int. The intermediate size of the MLP.
        mlp_bias: bool. Whether to use bias in MLP dense layers.
        layer_norm_epsilon: float. Epsilon for RMSNormalization.
        max_position_embeddings: int. Maximum sequence length for position
            embeddings. Defaults to 2048.
        rope_theta: float. The theta value for RoPE. Defaults to 10000.0.
        partial_rotary_factor: float. The factor for partial rotary embedding.
            Defaults to 1.0.
    """

    def __init__(
        self,
        hidden_size,
        num_attention_heads,
        num_key_value_heads,
        attention_bias,
        attention_dropout,
        rope_layer_enabled_list,
        layer_types,
        layer_idx,
        intermediate_size,
        mlp_bias,
        layer_norm_epsilon,
        max_position_embeddings=2048,
        rope_theta=10000.0,
        partial_rotary_factor=1.0,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.hidden_size = hidden_size
        self.layer_idx = layer_idx

        self.self_attn = SmolLM3Attention(
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            num_key_value_heads=num_key_value_heads,
            attention_bias=attention_bias,
            attention_dropout=attention_dropout,
            rope_layer_enabled_list=rope_layer_enabled_list,
            layer_types=layer_types,
            layer_idx=layer_idx,
            max_position_embeddings=max_position_embeddings,
            rope_theta=rope_theta,
            partial_rotary_factor=partial_rotary_factor,
            name="self_attn",
        )

        self.mlp = SmolLM3MLP(
            hidden_size=hidden_size,
            intermediate_size=intermediate_size,
            mlp_bias=mlp_bias,
            name="mlp",
        )

        self.input_layernorm = layers.RMSNormalization(
            epsilon=layer_norm_epsilon, axis=-1, name="input_layernorm"
        )
        self.post_attention_layernorm = layers.RMSNormalization(
            epsilon=layer_norm_epsilon, axis=-1, name="post_attention_layernorm"
        )

        self.attention_type = layer_types[layer_idx]

    def _compute_self_attention_mask(
        self,
        decoder_sequence,
        decoder_padding_mask,
        decoder_attention_mask,
        self_attention_cache,
        self_attention_cache_update_index,
    ):
        decoder_mask = merge_padding_and_attention_mask(
            decoder_sequence, decoder_padding_mask, decoder_attention_mask
        )
        batch_size = ops.shape(decoder_sequence)[0]
        input_length = output_length = ops.shape(decoder_sequence)[1]
        # We need to handle a rectangular causal mask when doing cached
        # decoding. For generative inference, `decoder_sequence` will
        # generally be length 1, and `cache` will be the full generation length.
        if self_attention_cache is not None:
            input_length = ops.shape(self_attention_cache)[2]

        cache_update_index = (
            0
            if self_attention_cache_update_index is None
            else self_attention_cache_update_index
        )

        causal_mask = compute_causal_mask(
            batch_size, input_length, output_length, cache_update_index
        )

        return (
            ops.minimum(decoder_mask, causal_mask)
            if decoder_mask is not None
            else causal_mask
        )

    def build(self, input_shape):
        """
        Builds the sub-layers based on the input shape.

        Args:
            input_shape: The input shape to the decoder layer
                (batch_size, seq_len, hidden_size).
        """
        # input_shape for SmolLM3DecoderLayer: (batch_size, seq_len,
        # hidden_size)
        batch_size = input_shape[0]
        seq_len = input_shape[1]

        head_dim = self.self_attn.head_dim
        pos_emb_shape = (batch_size, seq_len, head_dim)

        attn_mask_shape = (batch_size, 1, seq_len, seq_len)

        # Pass the correct input shape to self_attn's build method
        # The input_shape for self_attn.build is a list:
        # [hidden_states_shape, (pos_emb_shape, pos_emb_shape), attn_mask_shape]
        self.self_attn.build(
            [input_shape, (pos_emb_shape, pos_emb_shape), attn_mask_shape]
        )

        self.mlp.build(input_shape)
        self.input_layernorm.build(input_shape)
        self.post_attention_layernorm.build(input_shape)

        super().build(input_shape)

    def call(
        self,
        hidden_states,
        training=False,
        decoder_padding_mask=None,
        decoder_attention_mask=None,
        **kwargs,
    ):
        """
        Forward pass for SmolLM3DecoderLayer.

        Args:
            hidden_states: Input tensor of shape (batch_size,
                seq_len, hidden_size).
            position_embeddings: Optional tuple of (cos, sin)
                tensors for RoPE.
            training: Whether the layer is in training mode.
        """
        self_attention_cache = kwargs.get("self_attention_cache", None)
        self_attention_cache_update_index = kwargs.get(
            "self_attention_cache_update_index", None
        )

        self_attention_mask = self._compute_self_attention_mask(
            decoder_sequence=hidden_states,
            decoder_padding_mask=decoder_padding_mask,
            decoder_attention_mask=decoder_attention_mask,
            self_attention_cache=self_attention_cache,
            self_attention_cache_update_index=self_attention_cache_update_index,
        )

        residual = hidden_states
        hidden_states = self.input_layernorm(hidden_states)

        # Self Attention
        x = self.self_attn(
            hidden_states=hidden_states,
            training=training,
            attention_mask=self_attention_mask,
            **kwargs,
        )

        if isinstance(x, tuple):
            attn_output, self_attention_cache = x
        else:
            attn_output = x

        hidden_states = ops.add(residual, attn_output)

        residual = hidden_states
        hidden_states = self.post_attention_layernorm(hidden_states)
        hidden_states = self.mlp(hidden_states)
        hidden_states = ops.add(residual, hidden_states)

        if self_attention_cache is not None:
            return hidden_states, self_attention_cache
        else:
            return hidden_states

    def compute_output_shape(self, input_shape):
        """
        Computes the output shape of the layer.

        Args:
            input_shape: The input shape (batch_size, seq_len, hidden_size).

        Returns:
            The output shape, which is the same as the input shape:
            (batch_size, seq_len, hidden_size).
        """
        return input_shape


class SmolLM3RotaryEmbedding(layers.Layer):
    """Rotary Position Embedding (RoPE) layer for SmolLM3 model.

    Args:
        hidden_size: int. The hidden size of the model.
        num_attention_heads: int. The number of attention heads.
        max_position_embeddings: int. The maximum sequence length for position
            embeddings.
        rope_theta: float. The theta value for RoPE.
        partial_rotary_factor: float. The factor for partial rotary embedding.
    """

    def __init__(
        self,
        hidden_size,
        num_attention_heads,
        max_position_embeddings,
        rope_theta,
        partial_rotary_factor,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.max_position_embeddings = max_position_embeddings
        self.rope_theta = rope_theta
        self.partial_rotary_factor = partial_rotary_factor

        self.head_dim = self.hidden_size // self.num_attention_heads

        inv_freq_tensor, self.attention_scaling = rope_init(
            self.rope_theta, self.partial_rotary_factor, self.head_dim
        )

        self.inv_freq = self.add_weight(
            name="inv_freq",
            shape=ops.shape(inv_freq_tensor),
            dtype=inv_freq_tensor.dtype,
            initializer=initializers.Constant(
                ops.convert_to_numpy(inv_freq_tensor)
            ),
            trainable=False,  # This weight is not trained
        )
        self.original_inv_freq = self.inv_freq

    def build(self, input_shape):
        """
        Builds the layer. For SmolLM3RotaryEmbedding, this mainly
        ensures that the parent layer's build is called.

        Args:
            input_shape: A list/tuple of shapes for the inputs:
                [x_shape, position_ids_shape]
                - x_shape: (batch_size, ..., head_dim)
                - position_ids_shape: (batch_size, seq_len)
        """
        # No internal layers to explicitly build here, as inv_freq is
        # added in __init__
        super().build(input_shape)

    def call(
        self,
        x,
        start_index=0,
    ):
        """
        Forward pass for SmolLM3RotaryEmbedding.

        Args:
            x: Input tensor, typically query or key states.
               Shape can vary, but the last dimension is head_dim.
            position_ids: Tensor of position IDs of shape (batch_size, seq_len).
        """
        batch_size = ops.shape(x)[0]
        seq_len = ops.shape(x)[1]
        positions = ops.arange(seq_len, dtype="float32")
        positions = positions + ops.cast(start_index, dtype="float32")

        # inv_freq: (inv_freq_dim,) -> (1, inv_freq_dim, 1)
        # -> (batch, inv_freq_dim, 1)
        inv_freq_expanded = ops.expand_dims(
            ops.expand_dims(self.inv_freq, axis=0), axis=-1
        )
        inv_freq_expanded = ops.broadcast_to(
            inv_freq_expanded, (batch_size, ops.shape(self.inv_freq)[0], 1)
        )

        # positions: (seq_len,) -> (1, 1, seq_len)
        # -> (batch, 1, seq_len)
        position_ids_expanded = ops.expand_dims(
            ops.expand_dims(positions, axis=0), axis=0
        )
        position_ids_expanded = ops.broadcast_to(
            position_ids_expanded, (batch_size, 1, seq_len)
        )

        # matmul: (batch, inv_freq_dim, 1) @ (batch, 1, seq_len)
        # -> (batch, inv_freq_dim, seq_len)
        freqs = ops.matmul(
            ops.cast(inv_freq_expanded, "float32"),
            ops.cast(position_ids_expanded, "float32"),
        )

        # transpose: (batch, inv_freq_dim, seq_len) ->
        # (batch, seq_len, inv_freq_dim)
        freqs = ops.transpose(freqs, axes=(0, 2, 1))

        emb = ops.concatenate((freqs, freqs), axis=-1)

        cos = ops.cos(emb) * self.attention_scaling
        sin = ops.sin(emb) * self.attention_scaling

        return ops.cast(cos, x.dtype), ops.cast(sin, x.dtype)

    def compute_output_shape(self, input_shape):
        """
        Computes the output shape of the layer.

        Args:
            input_shape: A list/tuple of shapes for the inputs:
                         [x_shape, position_ids_shape]
                         - x_shape: (batch_size, ..., head_dim)
                         - position_ids_shape: (batch_size, seq_len)

        Returns:
            A list of output shapes for (cos, sin):
            [(batch_size, seq_len, head_dim), (batch_size, seq_len, head_dim)]
        """
        if input_shape[1] is not None and len(input_shape[1]) >= 2:
            batch_size = input_shape[1][0]
            seq_len = input_shape[1][1]
        else:
            # Fallback if position_ids_shape is None or malformed.
            # In this case, the batch_size and seq_len are unknown.
            batch_size = None
            seq_len = None

        # The output cos and sin have shape (batch_size, seq_len, head_dim)
        output_shape = (batch_size, seq_len, self.head_dim)

        return [output_shape, output_shape]



"""Qwen3 MoE model preset configurations."""

backbone_presets = {
    "qwen3_moe_30b_a3b_en": {
        "metadata": {
            "description": (
                " Mixture-of-Experts (MoE) model has 30.5 billion total"
                " parameters with 3.3 billion activated, built on 48 layers"
                " and utilizes 32 query and 4 key/value attention heads"
                " with 128 experts (8 active)."
            ),
            "params": 30532122624,
            "path": "qwen3_moe",
        },
        "kaggle_handle": "kaggle://keras/qwen-3-moe/keras/qwen3_moe_30b_a3b_en/2",
    },
    "qwen3_moe_235b_a22b_en": {
        "metadata": {
            "description": (
                " Mixture-of-Experts (MoE) model has 235 billion"
                " total parameters with 22 billion activated, built on 94"
                " layers and utilizes 64 query and 4 key/value attention heads"
                " with 128 experts (8 active)."
            ),
            "params": 235093634560,
            "path": "qwen3_moe",
        },
        "kaggle_handle": "kaggle://keras/qwen-3-moe/keras/qwen3_moe_235b_a22b_en/1",
    },
}



from keras import layers
from keras import ops


class FlowMatchEulerDiscreteScheduler(layers.Layer):
    """Flow-matching sampling euler scheduler.

    This layer is used to compute the discrete sigmas for the diffusion chain.
    Typically, the sigma refers to the amount of noise added during the
    diffusion process.

    Args:
        num_train_timesteps: int. The number of diffusion steps to train the
            model.
        shift: float. The shift value for the timestep schedule.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.

    Call arguments:
        inputs: The current step of the diffusion process.
        num_steps: The total number of steps in the diffusion process.

    References:
    - [Common Diffusion Noise Schedules and Sample Steps are Flawed](
    https://arxiv.org/abs/2305.08891).
    - [Scaling Rectified Flow Transformers for High-Resolution Image Synthesis](
    https://arxiv.org/abs/2403.03206).
    """

    def __init__(self, num_train_timesteps=1000, shift=3.0, **kwargs):
        super().__init__(**kwargs)
        self.num_train_timesteps = int(num_train_timesteps)
        self.shift = float(shift)

        timesteps = ops.linspace(
            1, num_train_timesteps, num_train_timesteps, dtype="float32"
        )
        timesteps = ops.flip(timesteps, axis=0)
        sigmas = self._timestep_to_sigma(timesteps)

        self.sigma_min = sigmas[-1]
        self.sigma_max = sigmas[0]

    def _sigma_to_timestep(self, sigma):
        return sigma * self.num_train_timesteps

    def _timestep_to_sigma(self, timestep):
        sigma = ops.divide(timestep, self.num_train_timesteps)
        if self.shift != 1.0:
            sigma = ops.divide(
                ops.multiply(self.shift, sigma),
                ops.add(1, ops.multiply(self.shift - 1.0, sigma)),
            )
        return sigma

    def set_sigmas(self, num_steps):
        timesteps = ops.linspace(
            self._sigma_to_timestep(self.sigma_max),
            self._sigma_to_timestep(self.sigma_min),
            num_steps,
        )
        sigmas = self._timestep_to_sigma(timesteps)
        sigmas = ops.concatenate([sigmas, ops.zeros((1,), dtype=sigmas.dtype)])
        self.sigmas = sigmas

    def call(self, inputs, num_steps):
        if not hasattr(self, "sigmas"):
            self.set_sigmas(num_steps)

        step = ops.expand_dims(
            ops.convert_to_tensor(inputs, dtype="int32"), axis=0
        )
        sigma = ops.take(self.sigmas, step)
        timestep = self._sigma_to_timestep(sigma)
        return sigma, timestep

    def add_noise(self, inputs, noises, step, num_steps):
        sigma, _ = self(step, num_steps)
        return ops.add(
            ops.multiply(sigma, noises),
            ops.multiply(ops.subtract(1.0, sigma), inputs),
        )

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_train_timesteps": self.num_train_timesteps,
                "shift": self.shift,
            }
        )
        return config

    def compute_output_shape(self):
        # Returns a tuple of (sigma, timestep).
        return (None,), (None,)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.roberta.roberta_text_classifier import (
    RobertaTextClassifier,  # noqa: E501
)
from keras_hub.src.models.roformer_v2.roformer_v2_backbone import (
    RoformerV2Backbone,
)
from keras_hub.src.models.roformer_v2.roformer_v2_text_classifier_preprocessor import (  # noqa: E501
    RoformerV2TextClassifierPreprocessor,
)


@keras_hub_export("keras_hub.models.RoformerV2TextClassifier")
class RoformerV2TextClassifier(RobertaTextClassifier):
    """An end-to-end RoformerV2 model for classification tasks.

    This model attaches a classification head to
    `keras_hub.model.RoformerV2Backbone`, mapping from the backbone outputs
    to logits suitable for a classification task. For usage of this model with
    pre-trained weights, use the `from_preset()` constructor.

    This model can optionally be configured with a `preprocessor` layer, in
    which case it will automatically apply preprocessing to raw inputs during
    `fit()`, `predict()`, and `evaluate()`. This is done by default when
    creating the model with `from_preset()`.

    Disclaimer: Pre-trained models are provided on an "as is" basis, without
    warranties or conditions of any kind.

    Args:
        backbone: A `keras_hub.models.RoformerV2Backbone` instance.
        num_classes: int. Number of classes to predict.
        preprocessor: A `keras_hub.models.RoformerV2TextClassifierPreprocessor`
            or `None`. If `None`, this model will not apply preprocessing, and
            inputs should be preprocessed before calling the model.
        activation: Optional `str` or callable. The
            activation function to use on the model outputs. Set
            `activation="softmax"` to return output probabilities.
            Defaults to `None`.
        dropout: float. The dropout probability value, applied after the dense
            layer.

    Examples:

    Raw string data.
    ```python
    features = ["The quick brown fox jumped.", "I forgot my homework."]
    labels = [0, 3]

    # Pretrained classifier.
    classifier = keras_hub.models.RoformerV2TextClassifier.from_preset(
        "roformer_v2_base_zh",
        num_classes=4,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    classifier.predict(x=features, batch_size=2)

    # Re-compile (e.g., with a new learning rate).
    classifier.compile(
        loss=keras.losses.SparseCategoricalCrossentropy(from_logits=True),
        optimizer=keras.optimizers.Adam(5e-5),
        jit_compile=True,
    )
    # Access backbone programmatically (e.g., to change `trainable`).
    classifier.backbone.trainable = False
    # Fit again.
    classifier.fit(x=features, y=labels, batch_size=2)
    ```

    Preprocessed integer data.
    ```python
    features = {
        "token_ids": np.ones(shape=(2, 12), dtype="int32"),
        "segment_ids": np.array([[0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0]] * 2),
        "padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]] * 2),
    }
    labels = [0, 3]

    # Pretrained classifier without preprocessing.
    classifier = keras_hub.models.RoformerV2TextClassifier.from_preset(
        "roformer_v2_base_zh",
        num_classes=4,
        preprocessor=None,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    ```

    Custom backbone and vocabulary.
    ```python
    features = ["The quick brown fox jumped.", "I forgot my homework."]
    labels = [0, 3]

    vocab = ["[UNK]", "[CLS]", "[SEP]", "[PAD]", "[MASK]"]
    vocab += ["The", "quick", "brown", "fox", "jumped", "."]
    tokenizer = keras_hub.models.RoformerV2Tokenizer(
        vocabulary=vocab,
    )
    preprocessor = keras_hub.models.RoformerV2TextClassifierPreprocessor(
        tokenizer=tokenizer,
        sequence_length=128,
    )
    backbone = keras_hub.models.RoformerV2Backbone(
        vocabulary_size=30552,
        num_layers=4,
        num_heads=4,
        hidden_dim=256,
        intermediate_dim=512,
        max_wavelength=128,
        head_size=64,
    )
    classifier = keras_hub.models.RoformerV2TextClassifier(
        backbone=backbone,
        preprocessor=preprocessor,
        num_classes=4,
    )
    classifier.fit(x=features, y=labels, batch_size=2)
    ```
    """

    backbone_cls = RoformerV2Backbone
    preprocessor_cls = RoformerV2TextClassifierPreprocessor



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.seq_2_seq_lm_preprocessor import Seq2SeqLMPreprocessor
from keras_hub.src.models.t5gemma.t5gemma_backbone import T5GemmaBackbone
from keras_hub.src.models.t5gemma.t5gemma_tokenizer import T5GemmaTokenizer
from keras_hub.src.utils.tensor_utils import preprocessing_function

try:
    import tensorflow as tf
except ImportError:
    tf = None


@keras_hub_export("keras_hub.models.T5GemmaSeq2SeqLMPreprocessor")
class T5GemmaSeq2SeqLMPreprocessor(Seq2SeqLMPreprocessor):
    """T5Gemma Seq2Seq LM preprocessor.

    This preprocessing layer is meant for use with
    `keras_hub.models.T5GemmaSeq2SeqLM`. By default, it will take in batches of
    strings, and return outputs in a `(x, y, sample_weight)` format, where the
    `y` label is the next token id in the `x` sequence.

    For use with generation, the layer also exposes two methods
    `generate_preprocess()` and `generate_postprocess()`. When this preprocessor
    is attached to a `keras_hub.models.T5GemmaSeq2SeqLM` instance, these methods
    will be called implicitly in `generate()`. They can also be called
    standalone (e.g. to precompute preprocessing inputs for generation in a
    separate process).

    Args:
        tokenizer: A `keras_hub.models.T5GemmaTokenizer` instance.
        encoder_sequence_length: The length of the packed encoder inputs.
        decoder_sequence_length: The length of the packed decoder inputs.
        add_start_token: If `True`, the preprocessor will prepend the
            tokenizer start token to each input sequence. For T5Gemma models,
            this should be `False`. Defaults to `False`.
        add_end_token: If `True`, the preprocessor will append the tokenizer end
            token to each input sequence. For T5Gemma models, this should be
            `True`. Defaults to `True`.

    Call arguments:
        x: A dictionary with two keys, `"encoder_text"` and `"decoder_text"`.
           The values can be a string, a `tf.Tensor` or a list of python
           strings.
        y: Label data. Should always be `None` as the layer generates labels.
        sample_weight: Label weights. Should always be `None` as the layer
            generates label weights.
        encoder_sequence_length: Pass to override the configured
            `encoder_sequence_length` of the layer.
        decoder_sequence_length: Pass to override the configured
            `decoder_sequence_length` of the layer.

    Examples:
    ```python
    import tensorflow as tf
    import numpy as np

    # Load the preprocessor from a preset.
    preprocessor = keras_hub.models.T5GemmaSeq2SeqLMPreprocessor.from_preset(
        "t5gemma_b_b_prefixlm_it"
    )

    # For example usage, see the dictionary example below which provides
    # both encoder and decoder text.
    # Tokenize a batch of sentences.
    preprocessor(["The quick brown fox jumped.", "Call me Ishmael."])
    # Tokenize a dictionary with separate encoder and decoder inputs.
    preprocessor({
        "encoder_text": "The quick brown fox jumped.",
        "decoder_text": "The fast fox."
    })

    # Apply tokenization to a `tf.data.Dataset`.
    encoder_features = tf.constant(["The quick brown fox.", "Call me Ishmael."])
    decoder_features = tf.constant(["The fast fox.", "I am Ishmael."])
    ds = tf.data.Dataset.from_tensor_slices(
        {"encoder_text": encoder_features, "decoder_text": decoder_features}
    )
    ds = ds.map(preprocessor, num_parallel_calls=tf.data.AUTOTUNE)

    # Prepare tokens for generation.
    preprocessor.generate_preprocess({
        "encoder_text": "The quick brown fox jumped.",
        "decoder_text": "The fast fox."
    })

    # Map generation outputs back to strings.
    preprocessor.generate_postprocess({
        'decoder_token_ids': np.array([[2, 714, 4320, 8426, 25341, 1, 0, 0]]),
        'decoder_padding_mask': np.array([[1, 1, 1, 1, 1, 1, 0, 0]]),
    })
    ```
    """

    backbone_cls = T5GemmaBackbone
    tokenizer_cls = T5GemmaTokenizer

    def __init__(
        self,
        tokenizer,
        encoder_sequence_length=512,
        decoder_sequence_length=512,
        add_start_token=False,
        add_end_token=True,
        **kwargs,
    ):
        # Do not pass `add_start_token` and `add_end_token` to the base class.
        super().__init__(
            tokenizer=tokenizer,
            encoder_sequence_length=encoder_sequence_length,
            decoder_sequence_length=decoder_sequence_length,
            **kwargs,
        )
        # Store them directly on the subclass instance.
        self.add_start_token = add_start_token
        self.add_end_token = add_end_token

    @preprocessing_function
    def call(
        self,
        x,
        y=None,
        sample_weight=None,
        *,
        encoder_sequence_length=None,
        decoder_sequence_length=None,
        sequence_length=None,
    ):
        if encoder_sequence_length is None:
            encoder_sequence_length = self.encoder_sequence_length
        decoder_sequence_length = decoder_sequence_length or sequence_length
        if decoder_sequence_length is None:
            decoder_sequence_length = self.decoder_sequence_length

        encoder_inputs = self.tokenizer(x["encoder_text"])
        encoder_token_ids, encoder_padding_mask = self.encoder_packer(
            encoder_inputs,
            sequence_length=encoder_sequence_length,
            add_start_value=self.add_start_token,
            add_end_value=self.add_end_token,
        )
        decoder_inputs = self.tokenizer(x["decoder_text"])
        decoder_token_ids, decoder_padding_mask = self.decoder_packer(
            decoder_inputs,
            sequence_length=decoder_sequence_length + 1,
            add_start_value=True,
            add_end_value=self.add_end_token,
        )
        x = {
            "encoder_token_ids": encoder_token_ids,
            "encoder_padding_mask": encoder_padding_mask,
            "decoder_token_ids": decoder_token_ids[..., :-1],
            "decoder_padding_mask": decoder_padding_mask[..., :-1],
        }
        y = decoder_token_ids[..., 1:]
        sample_weight = decoder_padding_mask[..., 1:]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)

    @preprocessing_function
    def generate_preprocess(
        self,
        x,
        *,
        encoder_sequence_length=None,
        decoder_sequence_length=None,
        sequence_length=None,
    ):
        if not self.built:
            self.build(None)

        if isinstance(x, dict):
            encoder_text = x["encoder_text"]
            decoder_text = x["decoder_text"]
        else:
            encoder_text = x
            decoder_text = tf.fill((tf.shape(encoder_text)[0],), "")

        if encoder_sequence_length is None:
            encoder_sequence_length = self.encoder_sequence_length
        decoder_sequence_length = decoder_sequence_length or sequence_length
        if decoder_sequence_length is None:
            decoder_sequence_length = self.decoder_sequence_length

        encoder_token_ids = self.tokenizer(encoder_text)
        encoder_token_ids, encoder_padding_mask = self.encoder_packer(
            encoder_token_ids,
            sequence_length=None,
            add_start_value=self.add_start_token,
            add_end_value=False,
        )

        decoder_token_ids = self.tokenizer(decoder_text)
        decoder_token_ids, decoder_padding_mask = self.decoder_packer(
            decoder_token_ids,
            sequence_length=decoder_sequence_length,
            add_start_value=True,
            add_end_value=False,
        )

        return {
            "encoder_token_ids": encoder_token_ids,
            "encoder_padding_mask": encoder_padding_mask,
            "decoder_token_ids": decoder_token_ids,
            "decoder_padding_mask": decoder_padding_mask,
        }

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "add_start_token": self.add_start_token,
                "add_end_token": self.add_end_token,
            }
        )
        return config
